import React, { useEffect, useState } from "react";
import { StyleSheet, View } from "react-native";
import PropTypes from "prop-types";
import API, { getEditions, getSpecials } from "./utils/api";
import { colors } from "./utils/colors";
import { CircleSnail } from "./components/Progress/progress-circle";
import Logger from "./utils/log";
import Network from "./utils/network";
import Errors from "./components/Error/errors-view";
import RoutesHelper from "./helpers/routes-helper";
import EditionsHelper from "./helpers/editions-helper";
import RouteSeparator from "./models/route-separator";
import RouteHeader from "./models/route-header";
import LiveConfig from "./data/live-config";
import PaperSetupConfig from "./data/paper-setup-config";
import LocaleConfig from "./data/locale-config";
import UserConfig from "./data/user-config";
import Content from "./Content";
import { TABS } from "./interfaces/navigations";
import { VERSION } from "./version";
import { getIssueAspectRatio } from "./utils/image";
import { AspectRatioContext } from "./components/Context/context";
import Issue from "./models/issue";
import { EventRegister } from "./events/manager";
import Events from "./events/events";

const styles = StyleSheet.create({
    app: {
        height: "100%",
        flex: 1,
        justifyContent: "center",
        alignItems: "center",
        backgroundColor: colors.white
    },
    errors: {
        position: "absolute",
        bottom: 0,
        left: 0,
        right: 0
    }
});

function initAPI() {
    API.setServer(LiveConfig.getServer());
    API.setEntryPoint(LiveConfig.getEntryPoint());
    API.setParams({
        ...LiveConfig.apiParams(),
        pSetup: LiveConfig.getPSetupName(),
        version: VERSION
    });
    API.init();
}

async function getStateData(getSavedIssues) {
    const editionsRaw = await getEditions({ pSetup: LiveConfig.getMainPSetupName() });
    LiveConfig.getSettings().setEditions(editionsRaw);
    let editions = EditionsHelper.parseEditions(LiveConfig.getSettings().editions);
    if (typeof getSavedIssues === "function") {
        try {
            const savedIssues = await getSavedIssues();
            EditionsHelper.updateDownloadIssues(editions, savedIssues);
            Logger.log(savedIssues);
        } catch (e) {
            Logger.warn(e.message);
        }
    }
    editions = EditionsHelper.filterEditions(editions, (issue: Issue) => Network.isOnline() || issue.isDownloaded());
    const firstIssue = EditionsHelper.getFirstValidIssue(editions);
    const aspectRatio = await getIssueAspectRatio(firstIssue);
    return {
        editions,
        aspectRatio
    };
}

function Container(props) {
    const {
        pSetup,
        server,
        pSetupOverride,
        WebViewComponent,
        initialRoute: initialRouteName,
        getSavedIssues,
        languagePackOverride,
        onPSetupError,
        onLocaleError,
        onUserSettingsError,
        onEditionsError,
        onSpecialsError,
        issueCallbacks,
        logCallbacks,
        urlParams,
        ...rest
    } = props;

    const [loading, setLoading] = useState(true);
    const [data, setData] = useState();

    const loadConfig = async () => {
        const startDate = new Date().getTime();
        LiveConfig.setInitialProps(props);
        LiveConfig.WebViewComponent = WebViewComponent;
        initAPI();
        Network.isHTTPS = LiveConfig.getProtocol() === "https:";
        let mPSetup, mLocale, mUserSettings;
        try {
            mPSetup = new PaperSetupConfig(pSetup);
            await mPSetup.download(pSetupOverride);
            await mPSetup.init();
        } catch (e) {
            onPSetupError(e);
        }
        try {
            mLocale = new LocaleConfig();
            await mLocale.init(mPSetup, languagePackOverride);
        } catch (e) {
            onLocaleError(e);
        }

        try {
            mUserSettings = new UserConfig();
            await mUserSettings.init(mPSetup);
        } catch (e) {
            onUserSettingsError(e);
        }

        LiveConfig.addProps({ mPSetup, mLocale, mUserSettings });
        Logger.debug(`Load config in: ${(new Date().getTime() - startDate) / 1000}s`);
    };

    const setRoutes = (editions) => {
        const { drawerRoutes: routes, tabsRoutes } = RoutesHelper.getUrlRoutes(LiveConfig.getPSetup().tabs);

        const mainRoute = RoutesHelper.getMainRoute(editions, tabsRoutes);
        const drawerRoutes = [mainRoute, ...routes];
        const tabsLabel = LiveConfig.getLocale().strings("str_tabs");
        if (tabsLabel) {
            drawerRoutes.unshift(new RouteHeader(tabsLabel), new RouteSeparator());
        }

        //if no initial route set open the first editions
        let initialRoute;
        if (!initialRouteName || initialRouteName === TABS.HOME) {
            initialRoute = mainRoute;
        } else {
            initialRoute = RoutesHelper.getInitialRoute(drawerRoutes, initialRouteName);
        }

        return { initialRoute, drawerRoutes };
    };

    const loadData = async () => {
        const startDate = new Date().getTime();
        const newData = await getStateData(getSavedIssues);
        const { initialRoute, drawerRoutes } = setRoutes(newData.editions);

        setData({
            drawerRoutes,
            ...newData,
            initialRoute: initialRoute ?? ""
        });
        Logger.debug(`Load data in: ${(new Date().getTime() - startDate) / 1000}s`);
    };

    const loadSpecials = async () => {
        try {
            const { editions } = data;
            const specials = EditionsHelper.parseSpecials(await getSpecials());
            setData((prevState) => ({
                ...prevState,
                specials: [...EditionsHelper.getShortuctEditions(editions), ...specials]
            }));
        } catch (e) {
            onSpecialsError(e);
        }
    };

    const load = async () => {
        await loadConfig();
        try {
            await loadData();
        } catch (e) {
            onEditionsError(e);
        }
        setLoading(false);
    };

    useEffect(() => {
        const onLogger = Logger.addListeners(logCallbacks);

        if (loading) {
            load();
        } else {
            loadSpecials();
        }
        Network.addConnectionChangeEventListener((state) => {
            Logger.debug(state);
        });
        return () => {
            if (onLogger) {
                onLogger.remove();
            }
            Network.removeConnectionChangeEventListener();
        };
    }, [loading]);

    useEffect(() => {
        if (data) {
            console.log(JSON.stringify(urlParams));
            const { editions } = data;
            if (editions) {
                LiveConfig.setUrlParams(urlParams);
                const { initialRoute, drawerRoutes } = setRoutes(editions);

                setData((prevState) => ({
                    ...prevState,
                    drawerRoutes,
                    initialRoute: initialRoute ?? ""
                }));
            }
        }
    }, [urlParams]);

    const remappedIssueCallbacks = {
        ...issueCallbacks,
        onIssuePress: (issue: Issue) => {
            const { xpaperUniversalLinks, remapEdition } = LiveConfig.getPSetup();
            const keys = Object.keys(xpaperUniversalLinks);

            const issuePSetup = issue.getPSetup();
            //detect if we are trying to open an xpaper that needs to be open externally
            if (keys.length > 0 && keys.includes(issuePSetup)) {
                //check for placeholders
                let link = xpaperUniversalLinks[issuePSetup];
                if (link) {
                    link = link.replace(/##issue##/g, issue.getIssue());
                    link = link.replace(/##edition##/g, issue.getEdition());
                    link = link.replace(/##pSetup##/g, issuePSetup);
                    EventRegister.emit(Events.ON_EXTERNAL_LINK_CLICK, { url: link });
                } else {
                    issueCallbacks.onIssuePress(issue);
                }
            } else {
                //check if the edition name has been remapped
                Object.keys(remapEdition).forEach((edition) => {
                    if (remapEdition[edition] === issue.getEdition()) {
                        issue.setEdition(edition);
                    }
                });

                issueCallbacks.onIssuePress(issue);
            }
        }
    };

    return (
        <View style={styles.app}>
            {!loading && data ? (
                <AspectRatioContext.Provider value={data.aspectRatio}>
                    <Content data={data} issueCallbacks={remappedIssueCallbacks} {...rest} />
                </AspectRatioContext.Provider>
            ) : (
                <CircleSnail />
            )}
            <Errors style={styles.errors} />
        </View>
    );
}

Container.propTypes = {
    apiParams: PropTypes.object,
    urlParams: PropTypes.object,
    pSetup: PropTypes.string.isRequired,
    server: PropTypes.string.isRequired,
    entryPoint: PropTypes.string,
    initialRoute: PropTypes.string,
    hideCloseButton: PropTypes.bool,
    onLocaleError: PropTypes.func.isRequired,
    onPSetupError: PropTypes.func.isRequired,
    onUserSettingsError: PropTypes.func.isRequired,
    onEditionsError: PropTypes.func.isRequired,
    onSpecialsError: PropTypes.func.isRequired,
    onRouteChange: PropTypes.func.isRequired,
    //could be used to filter edition list when device is offline
    getSavedIssues: PropTypes.func,
    WebViewComponent: PropTypes.any,
    issueCallbacks: PropTypes.shape({
        onIssuePress: PropTypes.func
    }),
    logCallbacks: PropTypes.shape({
        onLog: PropTypes.func,
        onDebug: PropTypes.func,
        onWarning: PropTypes.func,
        onError: PropTypes.func
    }),
    pSetupOverride: PropTypes.object,
    languagePackOverride: PropTypes.object
};

Container.defaultProps = {
    apiParams: {},
    urlParams: {},
    pSetupOverride: {},
    languagePackOverride: {},
    entryPoint: "/default_native_optionspage.php",
    hideCloseButton: false
};

export default Container;
