import { StyleSheet } from "react-native";
import LiveConfig from "../../data/live-config";
import Logger from "../../utils/log";
import { isAndroid, isIOS, isWeb, isWindows } from "../../utils/platform";
import { DEFAULT_STYLES } from "./css-default";
import { classIsSupported } from "./css-supported";

class CssHelper {
    /**
     * remove all keys mw from object
     * mw keys are related to screen width
     * @param {*} css
     * @returns
     */
    static #_maxWidthCss(css) {
        const mw = {};
        const rest = {};
        Object.keys(css).forEach((key) => {
            if (key.match(/mw[0-9]{2,}/)) {
                mw[key] = css[key];
            } else {
                rest[key] = css[key];
            }
        });
        return { mw, rest };
    }

    /**
     * return a style with all mw rules applied based on device width
     * mw means max-width so return all styles with mw greather than device width
     * @param {*} mw
     * @returns
     */
    static #_traverseAndFlattenMaxWidthCss(mw) {
        const wSize = LiveConfig.getWindow();
        let mwStyle = {};
        if (wSize && wSize.width) {
            const constraints = [];
            const mwKeys = Object.keys(mw);
            for (let i = 0; i < mwKeys.length; i++) {
                const constraint = parseInt(mwKeys[i].replace("mw", ""), 10);
                if (wSize.width <= constraint) {
                    constraints.push(`mw${constraint}`);
                }
            }
            constraints.forEach((constraint) => {
                const style = CssHelper.#_traverseAndFlattenCss(mw[constraint]);
                mwStyle = { ...mwStyle, ...style };
            });
        }
        return mwStyle;
    }

    /**
     * this method put all styles customized for a device (tablet|phone, ios|android, landscape|portrait) in a flat object
     *
     * "css-key": {
     *      "fontSize": 20,
     *      "tablet": {
     *          "backgroundColor": "red",
     *          "ios": {
     *              "height": 50,
     *              "landscape": {
     *                  "line-height": 10,
     *              },
     *          },
     *      },
     *      "ios": {
     *          "color": "red",
     *      },
     *      "landscape": {
     *          "flex": 1,
     *      }
     *  }
     * @param {*} css
     */
    static #_traverseAndFlattenCss(css) {
        if (css) {
            const { tablet, phone, ios, android, native, web, windows, landscape, portrait, ...cssRest } = css;
            const { mw, rest } = CssHelper.#_maxWidthCss(cssRest);

            let deviceStyle = {};
            if (LiveConfig.isTablet()) {
                deviceStyle = CssHelper.#_traverseAndFlattenCss(tablet);
            } else {
                deviceStyle = CssHelper.#_traverseAndFlattenCss(phone);
            }

            let maxWidthStyle = {};
            //?is usefull also inside mobile apps?
            if (isWeb()) {
                maxWidthStyle = CssHelper.#_traverseAndFlattenMaxWidthCss(mw);
            }

            let nativeStyle = {};
            if (!isWeb()) {
                nativeStyle = CssHelper.#_traverseAndFlattenCss(native);
            }

            let osStyle = {};
            if (isIOS()) {
                osStyle = CssHelper.#_traverseAndFlattenCss(ios);
            } else if (isAndroid()) {
                osStyle = CssHelper.#_traverseAndFlattenCss(android);
            } else if (isWeb()) {
                osStyle = CssHelper.#_traverseAndFlattenCss(web);
            } else if (isWindows()) {
                osStyle = CssHelper.#_traverseAndFlattenCss(windows);
            }

            let orientationStyle = {};
            if (LiveConfig.isPortrait()) {
                orientationStyle = CssHelper.#_traverseAndFlattenCss(portrait);
            } else {
                orientationStyle = CssHelper.#_traverseAndFlattenCss(landscape);
            }

            return {
                ...rest,
                ...deviceStyle,
                ...maxWidthStyle,
                ...nativeStyle,
                ...osStyle,
                ...orientationStyle
            };
        }
        return {};
    }

    static #_stylesArrayToObject(styles) {
        const style = {};
        styles.forEach((item) => {
            Object.keys(item).forEach((key) => {
                style[key] = item[key];
            });
        });
        return style;
    }

    /**
     * we manage the following style format
     * {
     *      "base": {
     *          "styles": {}
     *      },
     *      "themes": {
     *          "theme1": {
     *              "default": 1
     *          },
     *          "theme2": {
     *              "default": 0
     *          },
     *          "theme3": {
     *              "default": 0
     *          }
     *      }
     * }
     * @param {*} name
     * @param {*} theme
     */
    static #_getStyleByCssName(name) {
        const styles = [];
        //add default app styles
        styles.push(CssHelper.#_traverseAndFlattenCss(DEFAULT_STYLES[name]));
        if (!LiveConfig.getPSetup().disableRnStyles) {
            const rnStyles = CssHelper.#_getStyles();

            if (rnStyles && name) {
                const cssTheme = CssHelper.#_getTheme();
                const isDark = LiveConfig.isDark();

                if (!classIsSupported(name)) {
                    Logger.warn(`Css class ${name} is not supported`);
                }
                //we have a default style with all base styles
                let defaultStyles;
                //also user could select a specific style with some customization applied over default
                let userStyles;
                const { themes, base, baseLight, baseDark } = rnStyles;
                if (typeof themes === "object") {
                    Object.keys(themes).forEach((key) => {
                        //avoid server side issue, someone could add a key like a string that is not an object
                        if (typeof themes[key] === "object") {
                            //determine default theme
                            if (themes[key].default === 1) {
                                defaultStyles = themes[key];
                                //not override if user theme is set before default
                                if (!LiveConfig.getActiveTheme()) {
                                    LiveConfig.setActiveTheme(key);
                                }
                            }

                            if (cssTheme === "automatic") {
                                const isThemeDark = themes[key].dark === 1;
                                const isThemeAutomatic = themes[key].automatic === 1;
                                //if theme is dark and system is dark or theme is light and system is light
                                if (
                                    key !== "automatic" &&
                                    isThemeAutomatic &&
                                    ((isThemeDark && isDark) || (!isThemeDark && !isDark))
                                ) {
                                    userStyles = themes[key];
                                    LiveConfig.setActiveTheme(key);
                                }
                            } else if (cssTheme === key) {
                                //user style, could be undefined
                                userStyles = themes[key];
                                LiveConfig.setActiveTheme(key);
                            }
                        }
                    });
                }

                //than add style from base theme
                if (base && base.styles) {
                    styles.push(CssHelper.#_traverseAndFlattenCss(base.styles[name]));
                }

                const selectedStyles = userStyles || defaultStyles || {};
                //than add style from base appearance theme
                if (selectedStyles.dark === 1) {
                    if (baseDark && baseDark.styles) {
                        styles.push(CssHelper.#_traverseAndFlattenCss(baseDark.styles[name]));
                    }
                } else {
                    if (baseLight && baseLight.styles) {
                        styles.push(CssHelper.#_traverseAndFlattenCss(baseLight.styles[name]));
                    }
                }
                //add user theme if the user select something inside option page, otherwise add default selected theme
                if (selectedStyles && selectedStyles.styles) {
                    styles.push(CssHelper.#_traverseAndFlattenCss(selectedStyles.styles[name]));
                }
            }
        }
        return CssHelper.#_stylesArrayToObject(styles);
    }

    static #_getStylesByCssClasses(classes, attributes: { cssKey: String, active: Boolean, toggle: Boolean } = {}) {
        const newClasses = CssHelper.applyCustomClasses({ classes, ...attributes });
        if (Array.isArray(newClasses)) {
            return newClasses.map((css) => {
                return CssHelper.#_getStyleByCssName(css);
            });
        }
        return [];
    }

    static #_getTheme() {
        const { cssTheme } = LiveConfig.getCssTheme();
        return cssTheme;
    }

    static #_getStyles() {
        const { rnStyles } = LiveConfig.getPSetup().rnStyles;
        return rnStyles;
    }

    static #_getThemes() {
        if (LiveConfig.getPSetup().showThemesCustomization) {
            const { themes } = CssHelper.#_getStyles();
            if (themes) {
                return Object.keys(themes).map((key) => ({
                    ...themes[key],
                    key
                }));
            }
        }
        return [];
    }

    static applyCustomClasses(props) {
        const { classes, toggle, active, cssKey } = props;
        const newClasses = [];
        if (classes && classes.length) {
            newClasses.push(...classes);
            //add automatically active and inactive classes after all classes available
            if (toggle) {
                for (let i = 0; i < classes.length; i++) {
                    if (active) {
                        newClasses.push(`${classes[i]}-active`);
                    } else {
                        newClasses.push(`${classes[i]}-inactive`);
                    }
                }
            }
            //add automatically key custom classes after all classes available
            if (cssKey) {
                const { length } = newClasses;
                const fixedKey = cssKey.replace(/[\W_]+/g, "").toLowerCase();
                for (let i = 0; i < length; i++) {
                    newClasses.push(`${newClasses[i]}-${fixedKey}`);
                }
            }
        }
        return newClasses;
    }

    static getStyle(classes, attributes: { cssKey: String, active: Boolean, toggle: Boolean }) {
        const styles = CssHelper.#_getStylesByCssClasses(classes, attributes);
        return CssHelper.#_stylesArrayToObject(styles);
    }

    static flatten(styles = [], classes = []) {
        return StyleSheet.flatten([...styles, CssHelper.getStyle(classes)]);
    }

    static getStyleObjByCssClasses(classes, attributes: { cssKey: String, active: Boolean, toggle: Boolean }) {
        const styles = CssHelper.#_getStyleByCssClasses(classes, attributes);
        return CssHelper.#_stylesArrayToObject(styles);
    }
}

export default CssHelper;
