import React, { useCallback, useEffect, useState } from "react";
import { Dimensions } from "react-native";
import PropTypes from "prop-types";
import { NavigationContainer } from "@react-navigation/native";
import { createDrawerNavigator, getDrawerStatusFromState } from "@react-navigation/drawer";
import Route from "../../models/route";
import DrawerContent from "./drawer";
import NavigationConfig, { getActiveRouteFromState } from "../../data/navigation-config";
import MainHeader from "../Header/header";
import { DRAWER_WIDTH } from "../Css/common-styles";
import RouteUrl from "../../models/route-url";
import UrlPage from "../UrlPage/url-page";
import Publication from "../../models/publication";
import RouteTabs from "../../models/route-tabs";
import TabNavigator from "../Tab/tab-navigator";
import LiveConfig from "../../data/live-config";
import IssueItem from "../IssueItem/issue-item";

const Drawer = createDrawerNavigator();

/**
 * recursive function that return flat Drawer.Screen list from nested routes array
 * @param {*} routes
 * @returns
 */
function renderDrawerScreens({ routes, specials, props = {}, viewSize }) {
    const screens = [];
    routes.forEach((route) => {
        let Component = null;
        if (route instanceof RouteUrl) {
            Component = (
                <Drawer.Screen name={route.getRoute()} key={route.getRoute()}>
                    {(navProps) => <UrlPage url={route.getUrl()} {...props} {...navProps} />}
                </Drawer.Screen>
            );
        } else if (route instanceof RouteTabs) {
            Component = (
                <Drawer.Screen name={route.getRoute()} key={route.getRoute()}>
                    {(navProps) => (
                        <TabNavigator routes={route.getRoutes()} viewSize={viewSize} {...props} {...navProps} />
                    )}
                </Drawer.Screen>
            );
        }
        if (Component) {
            screens.push(
                Component,
                ...renderDrawerScreens({
                    routes: route.getRoutes(),
                    specials,
                    props
                })
            );
        }
    });
    return screens;
}

function getRouteByName(routes: Array<Route> = [], name) {
    let found;
    for (let i = 0; i < routes.length && !found; i++) {
        if (routes[i].getRoute() === name) {
            found = routes[i];
        } else {
            found = getRouteByName(routes[i].getRoutes(), name);
        }
    }
    return found;
}

/**
 * render custom drawer navigator
 * @param {*} props
 * @returns
 */
function DrawerNavigator(props) {
    const {
        routes,
        initialRoute,
        specials,
        onRouteChange,
        issueCallbacks,
        LogoComponent,
        onLogoPress,
        showCloseButton
    } = props;

    if (!routes.length) {
        return null;
    }

    const [windowDimensions, setWindowDimensions] = useState(Dimensions.get("window"));

    const navigatorStateChange = useCallback(
        ({ data }) => {
            const open = getDrawerStatusFromState(data.state);
            NavigationConfig.setState(data.state);
            NavigationConfig.setDrawerState(open === "open");
            if (typeof onRouteChange === "function") {
                onRouteChange(getRouteByName(routes, getActiveRouteFromState(data.state)));
            }
        },
        [onRouteChange]
    );

    useEffect(() => {
        const onDimensionsChange = Dimensions.addEventListener("change", ({ window: wDimensions }) => {
            setWindowDimensions(wDimensions);
        });

        return () => {
            if (onDimensionsChange) {
                onDimensionsChange.remove();
            }
        };
    }, []);

    const initialRouteName = initialRoute ? initialRoute.getRoute() : "";

    return (
        <>
            <MainHeader
                LogoComponent={LogoComponent}
                onLogoPress={onLogoPress}
                backgroundColor={LiveConfig.getForegroundColor()}
            />
            <NavigationContainer independent>
                <Drawer.Navigator
                    initialRouteName={initialRouteName}
                    useLegacyImplementation
                    screenOptions={{
                        //this line guarantee that one webview per time is rendered
                        unmountOnBlur: true,
                        headerShown: false,
                        drawerType: "slide", //"front"
                        drawerStyle: {
                            width: DRAWER_WIDTH
                        }
                    }}
                    screenListeners={{
                        state: navigatorStateChange
                    }}
                    drawerContent={(navProps) => {
                        NavigationConfig.setNavigation(navProps.navigation);
                        return <DrawerContent {...navProps} routes={routes} />;
                    }}>
                    {renderDrawerScreens({
                        routes,
                        specials,
                        props: { issueCallbacks },
                        viewSize: windowDimensions
                    })}
                </Drawer.Navigator>
            </NavigationContainer>
        </>
    );
}

DrawerNavigator.propTypes = {
    routes: DrawerContent.propTypes.routes,
    initialRoute: PropTypes.oneOfType([PropTypes.string, PropTypes.instanceOf(Route)]),
    specials: PropTypes.arrayOf(PropTypes.instanceOf(Publication)),
    onRouteChange: PropTypes.func.isRequired,
    issueCallbacks: IssueItem.propTypes.issueCallbacks,
    onLogoPress: PropTypes.func,
    LogoComponent: MainHeader.propTypes.LogoComponent,
    showCloseButton: PropTypes.bool
};

DrawerNavigator.defaultProps = {
    specials: [],
    showCloseButton: true
};

export default DrawerNavigator;
