import React, { useCallback, useState } from "react";
import PropTypes from "prop-types";
import { Platform, ScrollView, View } from "react-native";
import Edition from "../../models/edition";
import { commonStyles, ISSUE_MIN_WIDTH, MAIN_ISSUE_MAX_WIDTH, MAIN_VIEW_PADDING } from "../Css/common-styles";
import { Css } from "../Css/css";
import Issue from "../../models/issue";
import EditionsSecondary from "../EditionsSecondary/editions-secondary";
import IssueShadowItem from "../IssueItem/issue-shadow-item";
import styles from "./styles";
import EditionDayBg from "./edition-day-bg";
import IssueItem from "../IssueItem/issue-item";
import EditionsHelper from "../../helpers/editions-helper";
import IssuesGrid from "../Grid/issues-grid";
import LiveConfig from "../../data/live-config";
import { colors } from "../../utils/colors";
import EditionDayHeader from "./edition-day-header";
import IcPlus from "../Icon/ic-plus";

function getIssuesFromEditions(editions, refIssue) {
    const issues = [];
    editions.forEach((edition: Edition) => {
        const subEditions = edition.getGroup();
        if (subEditions.length > 0) {
            issues.push(...getIssuesFromEditions(subEditions, refIssue));
        } else {
            issues.push(...edition.getIssues().filter((issue: Issue) => issue.getIssue() === refIssue));
        }
    });

    return issues;
}

function getLastIssuesFromEditions(editions) {
    const issues = [];
    editions.forEach((edition: Edition) => {
        const subEditions = edition.getGroup();
        if (subEditions.length > 0) {
            issues.push(...getLastIssuesFromEditions(subEditions));
        } else {
            const editionIssues = edition.getIssues();
            if (editionIssues.length > 0) {
                issues.push(editionIssues[0]);
            }
        }
    });

    return issues;
}

/**
 * manage main edition and list with specials and other issue of selected edition
 * @param {*} props
 * @returns
 */
function EditionDayPage(props) {
    const { edition, issueDayIndex, issueCallbacks, aspectRatio, Header, horizontal, enableCalendarConstraint } = props;

    const [layout, setLayout] = useState();
    const [height, setHeight] = useState();

    const onLayout = useCallback(({ nativeEvent }) => {
        setLayout(nativeEvent.layout);
    });

    const onListLayout = useCallback(({ nativeEvent }) => {
        setHeight(nativeEvent.layout.height);
    });

    const [mainEdition, ...otherEditions] = edition.getGroup();
    const issues = mainEdition.getIssues();
    const mainIssue = issues[issueDayIndex];

    let Component = null;
    if (layout) {
        const headerPadding = horizontal ? MAIN_VIEW_PADDING : 0;
        const availableWith = horizontal ? layout.width / 2 : layout.width;
        const referenceWidth = Math.min(availableWith, MAIN_ISSUE_MAX_WIDTH);
        const availableHeight = layout.height - EditionDayHeader.HEIGHT;

        const mainThumbnailWidth = referenceWidth - MAIN_VIEW_PADDING * 2 - 10;
        const mainThumbnailHeight = mainThumbnailWidth / aspectRatio;

        const availableRightWidth = horizontal
            ? Math.max(0, layout.width - referenceWidth - MAIN_VIEW_PADDING)
            : mainThumbnailWidth;
        let otherThumbnailHeight = Math.min((mainThumbnailHeight * 3) / 5, availableHeight / 2);
        let otherThumbnailWidth = otherThumbnailHeight * aspectRatio;
        if (otherThumbnailWidth < ISSUE_MIN_WIDTH) {
            otherThumbnailWidth = ISSUE_MIN_WIDTH;
            otherThumbnailHeight = otherThumbnailWidth / aspectRatio;
        }

        const mainIssueHeight = Math.max(
            mainThumbnailHeight / 2,
            Math.min(mainThumbnailHeight, availableHeight - (horizontal ? 0 : otherThumbnailHeight))
        );
        const coloredAreaHeight = layout.height / 3;

        const secondaryEditions = EditionsHelper.getSecondaryEditions(otherEditions);
        //get inserts of the day, exclude secondary editions
        let dayOtherIssues = [];
        if (mainIssue) {
            const insertsEditions = otherEditions.filter((insert: Edition) => insert.isShortcut());
            dayOtherIssues = getIssuesFromEditions(insertsEditions, mainIssue.getIssue());

            //TODO display monthly edition near daily?
            //if no inserts associated to the day, check for placeholders if configured
            if (dayOtherIssues.length === 0) {
                const placeholderEditions = otherEditions.filter((insert: Edition) => insert.isPlaceholder());
                dayOtherIssues = getLastIssuesFromEditions(placeholderEditions);
            }
        }

        const heightStyle = {};
        if (height !== undefined) {
            //we need to update entire height, because the height is evaluated on top 0, list has negative top, so content is translated on top
            heightStyle.height = Platform.select({
                web: Math.max(0, height - coloredAreaHeight),
                default: height
            });
        }

        Component = (
            <Css classes={["opt-editions-scroll"]}>
                <ScrollView nestedScrollEnabled bounces={false} contentContainerStyle={[styles.container, heightStyle]}>
                    <EditionDayBg height={coloredAreaHeight} />
                    <View
                        style={[styles.list, horizontal && styles.listHorizontal, { top: -coloredAreaHeight }]}
                        onLayout={onListLayout}>
                        <View style={{ width: referenceWidth }}>
                            <View
                                style={{
                                    maxWidth: referenceWidth,
                                    width: availableRightWidth,
                                    paddingHorizontal: headerPadding,
                                    alignSelf: "center"
                                }}>
                                {Header}
                            </View>
                            {mainIssue ? (
                                <View
                                    style={[
                                        styles.other,
                                        styles.mainIssueContainer,
                                        {
                                            height: mainIssueHeight,
                                            maxWidth: MAIN_ISSUE_MAX_WIDTH
                                        }
                                    ]}>
                                    <IssueShadowItem
                                        item={mainIssue}
                                        style={[styles.mainIssue, { height: mainIssueHeight }]}
                                        imageStyle={[styles.mainIssueImage, { height: mainThumbnailHeight }]}
                                        imageParentStyle={[
                                            styles.mainIssueImageContainer,
                                            { width: mainThumbnailWidth, height: mainIssueHeight }
                                        ]}
                                        shadowStyle={[
                                            commonStyles.fullShadow,
                                            styles.mainIssueShadow,
                                            {
                                                height: mainIssueHeight
                                            }
                                        ]}
                                        showHighRes
                                        hideTitle
                                        issueCallbacks={issueCallbacks}
                                    />
                                </View>
                            ) : null}
                        </View>
                        <View
                            style={{
                                width: availableRightWidth
                            }}>
                            <EditionDayHeader.Container hide={!horizontal || !dayOtherIssues.length} />
                            <Css classes={["opt-editions-daily"]}>
                                <View style={[styles.other, horizontal && styles.otherHorizontal]}>
                                    <IssuesGrid
                                        data={dayOtherIssues.map((issue) => ({ issue }))}
                                        issueSize={{ width: otherThumbnailWidth, height: otherThumbnailHeight }}
                                        issueCallbacks={issueCallbacks}
                                    />
                                </View>
                            </Css>
                            <Css classes={["opt-editions-others"]}>
                                <View style={[styles.other, horizontal && styles.otherHorizontal]}>
                                    <EditionsSecondary
                                        mainEditionName={edition.getName()}
                                        editions={secondaryEditions}
                                        skipIssues={dayOtherIssues}
                                        issueCallbacks={issueCallbacks}
                                        closestIssue={enableCalendarConstraint && mainIssue ? mainIssue.getIssue() : ""}
                                        issueSize={{ width: otherThumbnailWidth, height: otherThumbnailHeight }}
                                        headerColor={
                                            !horizontal || dayOtherIssues.length
                                                ? colors.black
                                                : LiveConfig.getForegroundColor()
                                        }
                                    />
                                </View>
                            </Css>
                        </View>
                    </View>
                </ScrollView>
            </Css>
        );
    }

    return (
        <View style={commonStyles.container} onLayout={onLayout}>
            {Component}
        </View>
    );
}

EditionDayPage.propTypes = {
    Header: PropTypes.any,
    enableCalendarConstraint: PropTypes.bool,
    edition: PropTypes.instanceOf(Edition),
    issueDayIndex: PropTypes.number,
    issueCallbacks: IssueItem.propTypes.issueCallbacks,
    aspectRatio: PropTypes.number,
    horizontal: PropTypes.bool
};

EditionDayPage.defaultProps = {
    issueDayIndex: 0,
    Header: null,
    horizontal: false,
    enableCalendarConstraint: false
};

export default EditionDayPage;
