import React, { useCallback, useState } from "react";
import PropTypes from "prop-types";
import { View, StyleSheet, TouchableOpacity, Platform } from "react-native";
import Issue from "../../models/issue";
import { getGraphUrl, getLowUrl, getTextUrl } from "../../utils/api";
import { commonStyles } from "../Css/common-styles";
import { Css } from "../Css/css";
import TaImageIcon from "../Tecnavia/ta-image-icon";
import TaText from "../Tecnavia/ta-text";

const THUMBNAIL_HEIGHT = Platform.select({
    native: 150,
    web: 200
});

const THUMBNAIL_HR_THRESHOLD = 250;
const THUMBNAIL_MAIN_HEIGHT = 600;

const styles = StyleSheet.create({
    imageText: {},
    footer: {
        height: 20,
        justifyContent: "center"
    },
    footerText: {
        textAlign: "center"
    }
});

/**
 * update size of item by appling ration and constraints
 * @returns  {width,height}
 */
function updateSize({ height, aspectRatio, constraints }) {
    const defaultHeight = Math.min(height, constraints.height);
    const size = {
        width: defaultHeight * aspectRatio,
        height: defaultHeight
    };
    const scaleX = constraints.width / size.width;
    if (scaleX < 1) {
        size.width *= scaleX;
        size.height *= scaleX;
    }
    return size;
}

function editionItemKey(item, index) {
    return `${item.getIssue()}_${index}`;
}

/**
 * return issue images
 * could be low.jpg or high res by overriding graph.gpj with text.gif
 * @param {*} props
 * @returns
 */
function EditionItem(props) {
    const { item, useLow, style, imageStyle, issueCallbacks }: { item: Issue } = props;
    const [showHighRes, setShowHighRes] = useState(!useLow);
    const { onIssuePress } = issueCallbacks;
    const onPress = useCallback(() => {
        if (typeof onIssuePress === "function") {
            onIssuePress(item);
        }
    }, []);

    const onLayout = useCallback(
        ({ nativeEvent }) => {
            const { layout } = nativeEvent;
            if (layout.height > THUMBNAIL_HR_THRESHOLD && !showHighRes) {
                setShowHighRes(true);
            }
        },
        [showHighRes]
    );

    if (!item) {
        return null;
    }

    const imageProps = {
        resizeMode: "contain"
    };

    return (
        <TouchableOpacity style={style} onPress={onPress} collapsable={false} onLayout={onLayout}>
            <Css classes={["opt-issue-image-container"]}>
                <View style={commonStyles.container}>
                    {!showHighRes ? (
                        <Css classes={["opt-issue-image", "opt-issue-image-low"]}>
                            <TaImageIcon
                                style={[commonStyles.full, imageStyle]}
                                source={{
                                    uri: getLowUrl(item)
                                }}
                                {...imageProps}
                            />
                        </Css>
                    ) : (
                        <>
                            <Css classes={["opt-issue-image", "opt-issue-image-graph"]}>
                                <TaImageIcon
                                    style={[commonStyles.full, imageStyle]}
                                    source={{ uri: getGraphUrl(item) }}
                                    {...imageProps}
                                />
                            </Css>
                            <Css classes={["opt-issue-image", "opt-issue-image-text"]}>
                                <TaImageIcon
                                    style={[commonStyles.full, styles.imageText, imageStyle]}
                                    source={{ uri: getTextUrl(item) }}
                                    {...imageProps}
                                />
                            </Css>
                        </>
                    )}
                </View>
            </Css>
            <Css classes={["opt-issue-footer"]}>
                <View style={styles.footer}>
                    <Css classes={["opt-issue-footer-text"]}>
                        <TaText style={styles.footerText}>{item.getDateLabel()}</TaText>
                    </Css>
                </View>
            </Css>
        </TouchableOpacity>
    );
}

EditionItem.propTypes = {
    item: PropTypes.instanceOf(Issue).isRequired,
    useLow: PropTypes.bool,
    style: PropTypes.oneOfType([PropTypes.number, PropTypes.object, PropTypes.array]),
    imageStyle: PropTypes.oneOfType([PropTypes.number, PropTypes.object, PropTypes.array]),
    issueCallbacks: PropTypes.shape({
        onIssuePress: PropTypes.func
    })
};

EditionItem.defaultProps = {
    useLow: true,
    style: commonStyles.container,
    issueCallbacks: {}
};

export { THUMBNAIL_HEIGHT, THUMBNAIL_MAIN_HEIGHT, editionItemKey, updateSize };
export default EditionItem;
