import React, { useCallback, useState } from "react";
import PropTypes from "prop-types";
import { ScrollView, StyleSheet, View } from "react-native";
import Edition from "../../models/edition";
import IssueList from "../List/issue-list";
import EditionMain from "../EditionMain/edition-main";
import EditionsHelper from "../../helpers/editions-helper";
import { commonStyles } from "../Css/common-styles";
import { Css } from "../Css/css";
import Publication from "../../models/publication";
import Issue from "../../models/issue";
import EditionsSecondary from "../EditionsSecondary/editions-secondary";
import CssHelper from "../Css/css-methods";
import CssStyle from "../Css/css-styles";
import { THUMBNAIL_MAIN_HEIGHT, updateSize } from "../List/list-utils";
import LiveConfig from "../../data/live-config";
import IssueItem from "../IssueItem/issue-item";

const styles = StyleSheet.create({
    container: {
        flex: 1
    },
    hContainer: {
        flexDirection: "row"
    },
    other: {},
    hOther: {
        flex: 1
    }
});

/**
 * manage main edition and list with specials and other issue of selected edition
 * @param {*} props
 * @returns
 */
function EditionPage(props) {
    const { edition, specials, secondaryEditions, issueCallbacks, navigation, aspectRatio } = props;
    const [mainIssue, ...issues] = edition.getIssues();

    const [layout, setLayout] = useState();
    const onLayout = useCallback(({ nativeEvent }) => {
        setLayout(nativeEvent.layout);
    });

    /**
     * override issue press to have different behaviour instead of opening always the issue
     * @param {*} issue
     */
    const onIssuePress = (issue: Issue) => {
        //shortcut item navigate to drawer section instead of open the issue
        const found = EditionsHelper.getPublicationFromIssue(specials, issue);
        if (found && found.isShortcut()) {
            navigation.navigate(EditionsHelper.getRouteName(found.getName()));
        } else if (typeof issueCallbacks.onIssuePress === "function") {
            issueCallbacks.onIssuePress(issue);
        }
    };

    let ToRender = null;
    if (layout) {
        const cssContainer = CssHelper.flatten([commonStyles.container, styles.container], ["opt-editions-scroll"]);
        //update constraints to set correct item size on window resize
        const constraints = CssStyle.updateSizeByApplingStyle(layout, cssContainer);
        const size = updateSize({
            height: LiveConfig.getAvailableHeight(THUMBNAIL_MAIN_HEIGHT),
            aspectRatio,
            constraints
        });

        ToRender = (
            <View style={commonStyles.container}>
                <Css classes={["opt-editions-scroll"]}>
                    <ScrollView nestedScrollEnabled>
                        <View style={styles.container}>
                            <View collapsable={false}>
                                <EditionMain issue={mainIssue} issueCallbacks={issueCallbacks} size={size} />
                            </View>
                            <Css classes={["opt-editions-others"]}>
                                <View style={[styles.other, styles.hOther]}>
                                    <View collapsable={false}>
                                        <IssueList
                                            issues={issues}
                                            title={edition.getLabel()}
                                            issueCallbacks={issueCallbacks}
                                            horizontal
                                        />
                                    </View>
                                    <View collapsable={false}>
                                        <EditionsSecondary
                                            mainEditionName={edition.getName()}
                                            editions={secondaryEditions}
                                            issueCallbacks={{
                                                ...issueCallbacks,
                                                onIssuePress
                                            }}
                                            horizontal
                                        />
                                    </View>
                                </View>
                            </Css>
                        </View>
                    </ScrollView>
                </Css>
            </View>
        );
    }

    return (
        <View style={styles.container} onLayout={onLayout}>
            {ToRender}
        </View>
    );
}

EditionPage.propTypes = {
    navigation: PropTypes.object,
    aspectRatio: PropTypes.number,
    edition: PropTypes.instanceOf(Edition).isRequired,
    secondaryEditions: EditionsSecondary.propTypes.editions,
    specials: PropTypes.arrayOf(PropTypes.instanceOf(Publication)),
    issueCallbacks: IssueItem.propTypes.issueCallbacks
};

Edition.defaultProps = {
    maxWidth: 0,
    aspectRatio: 1
};

export default EditionPage;
