import React, { useState } from "react";
import PropTypes from "prop-types";
import { Modal, ScrollView, StyleSheet, View } from "react-native";
import { colors } from "../../utils/colors";
import { commonStyles, DRAWER_PERMANENT } from "../Css/common-styles";
import LiveConfig from "../../data/live-config";
import Edition from "../../models/edition";
import EditionsSecondaryModalHeader from "./editions-secondary-modal-header";
import EditionsSecondaryModalItem from "./editions-secondary-modal-item";
import { SearchTextContext, SelectedEditionsContext } from "../../data/user-config";
import { Css } from "../Css/css";
import JSUtils from "../../utils/js-utils";
import { SearchInput } from "../Search/search-input";

const MARGIN = 10;

const styles = StyleSheet.create({
    modal: {
        margin: 0
    },
    container: {
        flex: 1,
        backgroundColor: colors.transparent600,
        alignItems: "center",
        padding: 50
    },
    content: {
        padding: 0,
        backgroundColor: "white",
        flex: 1,
        margin: MARGIN
    },
    touchable: {
        height: 100,
        width: 100,
        justifyContent: "center",
        alignItems: "center"
    }
});

function filterEditions(editions = [], filter = "") {
    if (filter) {
        return editions.reduce((prev, current) => {
            if (current.getName().toLowerCase().search(filter.toLowerCase()) >= 0) {
                prev.push(current.getName());
            } else {
                const group = current.getGroup();
                if (group && group.length > 0) {
                    const filteredEditions = filterEditions(group, filter);
                    if (filteredEditions && filteredEditions.length > 0) {
                        prev.push(current.getName(), ...filteredEditions);
                    }
                }
            }

            return prev;
        }, []);
    }
    //return all editions
    /* return editions.reduce((prev, current) => {
        prev.push(current.getName());
        const group = current.getGroup();
        if (group && group.length > 0) {
            const filteredEditions = filterEditions(group);
            if (filteredEditions && filteredEditions.length > 0) {
                prev.push(current.getName(), ...filteredEditions);
            }
        }

        return prev;
    }, []);*/
    return undefined;
}

/**
 * modal that display an editions list to select preferred editions
 * @param {*} props
 * @returns
 */
function EditionsSecondaryModal(props) {
    const { onClose, visible, editions = [] } = props;
    const [selectedEditions, setSelectedEditions] = useState([...LiveConfig.getSettings().selectedEditions]);
    const [searchText, setSearchText] = useState("");

    const onEditionPress = (edition: Edition) => {
        const index = selectedEditions.indexOf(edition.getName());
        if (index > -1) {
            selectedEditions.splice(index, 1);
        } else {
            selectedEditions.push(edition.getName());
        }
        setSelectedEditions([...selectedEditions]);
    };

    const onHide = () => {
        setSearchText("");
        setSelectedEditions([...LiveConfig.getSettings().selectedEditions]);
        if (typeof onClose === "function") {
            onClose();
        }
    };

    const onSave = () => {
        LiveConfig.getSettings().setSelectedEditions(selectedEditions);
        onHide();
    };

    const filteredEditions = filterEditions(editions, searchText);

    const somethingChange = !JSUtils.areEqualShallow(selectedEditions, LiveConfig.getSettings().selectedEditions);

    return (
        <Css classes={["opt-editions-secondary-modal"]}>
            <Modal
                animationType="fade"
                onRequestClose={onHide}
                onDismiss={onHide}
                visible={visible}
                transparent
                style={styles.modal}>
                <Css classes={["opt-editions-secondary-container"]}>
                    <View style={styles.container}>
                        <Css classes={["opt-editions-secondary-content"]}>
                            <View
                                style={[
                                    commonStyles.card,
                                    styles.content,
                                    { width: Math.min(DRAWER_PERMANENT, LiveConfig.getWindow().width) - MARGIN * 2 }
                                ]}>
                                <EditionsSecondaryModalHeader
                                    title={LiveConfig.getLocale().strings("str_modal_header_preferred_edition")}
                                    onClose={onHide}
                                    onSave={somethingChange ? onSave : undefined}
                                    useDoneButton
                                />
                                <SearchInput
                                    style={{
                                        backgroundColor: colors.grey100,
                                        borderRadius: 20,
                                        borderColor: colors.white
                                    }}
                                    placeholder={LiveConfig.getLocale().strings("str_search")}
                                    onChangeText={(text) => {
                                        setSearchText(text);
                                    }}
                                />
                                <SelectedEditionsContext.Provider value={selectedEditions}>
                                    <SearchTextContext.Provider value={filteredEditions}>
                                        <Css classes={["opt-editions-secondary-body"]}>
                                            <ScrollView>
                                                {editions.map((edition, index) => (
                                                    <EditionsSecondaryModalItem
                                                        key={`edition_modal_item_${index}`}
                                                        edition={edition}
                                                        onPress={onEditionPress}
                                                    />
                                                ))}
                                            </ScrollView>
                                        </Css>
                                    </SearchTextContext.Provider>
                                </SelectedEditionsContext.Provider>
                            </View>
                        </Css>
                    </View>
                </Css>
            </Modal>
        </Css>
    );
}

EditionsSecondaryModal.propTypes = {
    onClose: PropTypes.func.isRequired,
    visible: PropTypes.bool,
    editions: PropTypes.arrayOf(PropTypes.instanceOf(Edition))
};

EditionsSecondaryModal.defaultProps = {
    visible: false
};

export default EditionsSecondaryModal;
