import React, { useCallback, useState } from "react";
import PropTypes from "prop-types";
import { View } from "react-native";
import IssueList from "../List/issue-list";
import Edition from "../../models/edition";
import EditionsHelper from "../../helpers/editions-helper";
import EditionsSecondaryEmpty from "./editions-secondary-empty";
import EditionsSecondaryModal from "./editions-secondary-modal";
import LiveConfig from "../../data/live-config";
import HeaderList from "../List/header-list";
import IcBookmarkPlus from "../Icon/ic-bookmark-plus";
import List from "../List/list";
import Issue from "../../models/issue";
import { issueItemKey } from "../IssueItem/issue-item";
import TaText from "../Tecnavia/ta-text";
import { commonStyles } from "../Css/common-styles";
import IssueCardItem from "../IssueItem/issue-card-item";
import { MAIN_VIEW_PADDING } from "../EditionDayPage/edition-day-page";

/**
 * display horizontal list of secondary editions
 * display secondary editions add button
 * @param {*} props
 * @returns
 */
function EditionsSecondary(props) {
    const { editions, mainEditionName, horizontal, forceShortcut, issueSize, ...rest } = props;
    const [showModal, setShowModal] = useState(false);

    if (!editions || editions.length === 0) {
        return null;
    }

    const { selectedEditions } = LiveConfig.getSettings();
    //const nothingSelected = selectedEditions.length === 0;

    const show = useCallback(() => {
        setShowModal(true);
    });
    const hide = useCallback(() => {
        setShowModal(false);
    });

    let Component = null;
    if (horizontal) {
        Component = (
            <>
                <View>
                    {selectedEditions.map((editionName: String, index) => {
                        if (editionName === mainEditionName) {
                            return null;
                        }
                        const edition: Edition = EditionsHelper.getEditionByName(editions, editionName);
                        return edition ? (
                            <IssueList
                                {...rest}
                                key={`selected_edition_${editionName}_${index}`}
                                title={edition.getLabel()}
                                issues={EditionsHelper.getIssuesFromSpecials([edition], mainEditionName, true)}
                                HeaderComponent={
                                    <HeaderList
                                        title={edition.getLabel()}
                                        Icon={<IcBookmarkPlus width={20} height={20} />}
                                    />
                                }
                                showTitle={forceShortcut || edition.isShortcut()}
                                horizontal
                            />
                        ) : null;
                    })}
                </View>
                <EditionsSecondaryEmpty onPress={show} />
            </>
        );
    } else {
        const { issueCallbacks } = props;

        const [layout, setLayout] = useState();

        const onLayout = ({ nativeEvent }) => {
            setLayout(nativeEvent.layout);
        };

        let issues = [];
        if (selectedEditions) {
            //if vertical prepare a unique vertical list with all the secondary editions
            const secondaryEditions = selectedEditions.reduce((current, editionName: String) => {
                if (editionName === mainEditionName) {
                    return null;
                }
                const edition: Edition = EditionsHelper.getEditionByName(editions, editionName);
                if (edition) {
                    current.push(edition);
                }
                return current;
            }, []);

            issues = EditionsHelper.getIssuesFromSpecials(secondaryEditions, mainEditionName, true);
        }

        const plusIssue = new Issue();
        plusIssue.setIssue("plus");
        plusIssue.setEdition("plus");

        let ListComponent = null;
        if (layout) {
            const showPlusAtBottom = layout.width < 450;

            const data = issues;
            if (!showPlusAtBottom) {
                data.push(plusIssue);
            }

            ListComponent = (
                <>
                    <List
                        {...rest}
                        key="selected_secondary_editions"
                        data={data}
                        horizontal={false}
                        keyExtractor={issueItemKey}
                        itemSize={issueSize}
                        gridMode
                        listContainerStyle={{ margin: 0 }}
                        renderItem={(itemProps) => {
                            const { item } = itemProps;
                            if (item.getIssue() === "plus" && item.getEdition() === "plus") {
                                return <EditionsSecondaryEmpty onPress={show} />;
                            }
                            return (
                                <IssueCardItem
                                    {...itemProps}
                                    style={[
                                        commonStyles.container,
                                        { marginHorizontal: MAIN_VIEW_PADDING / 5, marginVertical: 20 }
                                    ]}
                                    issueCallbacks={issueCallbacks}
                                    showHighRes
                                />
                            );
                        }}
                    />
                    {showPlusAtBottom ? <EditionsSecondaryEmpty onPress={show} /> : null}
                </>
            );
        }

        Component = (
            <View
                onLayout={onLayout}
                style={{
                    width: "100%",
                    maxWidth: 800
                }}>
                <TaText style={{ fontSize: 20, marginHorizontal: MAIN_VIEW_PADDING / 5 }}>
                    {LiveConfig.getLocale().strings("str_modal_header_preferred_edition")}
                </TaText>
                {ListComponent}
            </View>
        );
    }

    return (
        <>
            {Component}
            <EditionsSecondaryModal editions={editions} onClose={hide} visible={showModal} />
        </>
    );
}

EditionsSecondary.defaultProps = {
    issueSize: {
        width: 150,
        height: 200
    }
};

EditionsSecondary.propTypes = {
    mainEditionName: PropTypes.string,
    editions: PropTypes.arrayOf(PropTypes.instanceOf(Edition)),
    forceShortcut: PropTypes.bool,
    issueSize: PropTypes.object,
    ...IssueList.propTypes
};

export default EditionsSecondary;
