import React, { useRef, useState } from "react";
import PropTypes from "prop-types";
import { editionItemKey } from "../EditionItem/edition-item";
import LayoutList from "./layout-list";
import Edition from "../../models/edition";
import EditionsHelper from "../../helpers/editions-helper";
import IssueItem, { issueItemKey } from "../IssueItem/issue-item";
import Issue from "../../models/issue";
import EditionBackItem from "../EditionItem/edition-back-item";
import IssueCardItem from "../IssueItem/issue-card-item";
import { AspectRatioContext } from "../Context/context";
import { THUMBNAIL_HR_THRESHOLD } from "./list-utils";
import { EditionFolderItem } from "../EditionItem/edition-folder-item";
import { getEditionsPics } from "../../utils/api";
import LiveConfig from "../../data/live-config";
import { EditionFavoriteItem } from "../EditionItem/edition-favorite-item";

function keyExtractor(item, index) {
    if (item instanceof Edition) {
        return editionItemKey(item, index);
    }
    if (item instanceof Issue) {
        return issueItemKey(item, index);
    }
    return `edition_list_item_${index}`;
}

function EditionListItem(props) {
    const { item, index, size, onlyGroup, ...rest } = props;
    const { parent, issueCallbacks, onItemPress, onBackPress, showHighRes, onFavoritePress, isSelected } = rest;

    const issueProps = {};

    if (item instanceof Edition) {
        const isBackItem = parent && parent.getName() === item.getName();
        if (isBackItem) {
            return <EditionBackItem key={`back_item_${index}`} item={parent} onItemPress={onBackPress} />;
        }
        const issues = item.getIssues();
        if (issues.length > 1 || item.isLazy() || !issues.length || onlyGroup) {
            const enableFavorite = typeof onFavoritePress === "function";
            const EditionComponent =
                item.getGroup().length || !enableFavorite ? EditionFolderItem : EditionFavoriteItem;
            return (
                <EditionComponent
                    key={`edition_item_${index}`}
                    item={item}
                    onFavoriteIconPress={() => onFavoritePress(item)}
                    onFolderIconPress={() => onItemPress(item)}
                    showHighRes={showHighRes || size.height > THUMBNAIL_HR_THRESHOLD}
                    enableFavorite={enableFavorite}
                    selected={typeof isSelected === "function" ? isSelected(item) : false}
                />
            );
        }
        issueProps.item = issues[0];
    }
    return (
        <IssueCardItem
            key={`issue_item_${index}`}
            item={item}
            issueCallbacks={issueCallbacks}
            showHighRes={showHighRes || size.height > THUMBNAIL_HR_THRESHOLD}
            {...issueProps}
        />
    );
}

/**
 * this list could render issue or edition item
 * render issue if the selected edition doesn't contains nested edition
 * evaluate also item size by checking first issue aspect ratio and parent constraint
 * @param {*} props
 * @returns
 */
function EditionList(props) {
    const { horizontal, edition, issueCallbacks, showHighRes, onFavoritePress, isSelected, onlyGroup, ...rest } = props;
    const [data, setData] = useState({ items: edition.getGroup(), parent: undefined });
    const parentEditionName = useRef("");

    const updateData = (newData) => {
        setData(newData);
        parentEditionName.current = newData.parent ? newData.parent.getName() : "";
    };

    const lazyLoad = async (item) => {
        if (item.isLazy()) {
            const issue: Issue = EditionsHelper.getFirstValidIssue([item]);
            let pSetup = item.getPSetup() || issue.getPSetup();
            if (!item.isMain()) {
                pSetup = `${LiveConfig.getMainPSetupName()}-xp-${pSetup}`;
            }
            const pics = await getEditionsPics({
                pSetup
            });
            EditionsHelper.populateIssueOnEditions([edition], pics);
            if (item.getName() === parentEditionName.current) {
                updateData({ items: item.getIssues(), parent: item });
            }
        }
    };

    const onItemPress = (item: Edition) => {
        if (item.getGroup().length) {
            setData({ items: item.getGroup(), parent: item });
        } else if (!onlyGroup) {
            lazyLoad(item);
            updateData({ items: item.getIssues(), parent: item });
        }
    };

    const onBackPress = (item: Edition) => {
        const { items, parent } = EditionsHelper.getPreviousLevel(edition, data.parent);
        updateData({ items, parent });
    };

    const editions = [];
    //if parent add back button
    if (data.parent) {
        //ad fake edition for back button
        const backButtonEdition = new Edition();
        backButtonEdition.setName(`${data.parent.getName()}_back`);
        editions.push(backButtonEdition);
        //if parent has its own pages add it
        const parentIssues = data.parent.getIssues();
        if (parentIssues && parentIssues.length > 0) {
            editions.push(data.parent);
        }
    }
    editions.push(...data.items);

    return (
        <AspectRatioContext.Consumer>
            {(value) => (
                <LayoutList
                    aspectRatio={value}
                    horizontal={horizontal}
                    data={editions}
                    keyExtractor={keyExtractor}
                    renderItem={(itemProps) => (
                        <EditionListItem
                            {...itemProps}
                            issueCallbacks={issueCallbacks}
                            parent={data.parent}
                            showHighRes={showHighRes}
                            isSelected={isSelected}
                            onItemPress={onItemPress}
                            onBackPress={onBackPress}
                            onFavoritePress={onFavoritePress}
                            onlyGroup={onlyGroup}
                        />
                    )}
                    {...rest}
                />
            )}
        </AspectRatioContext.Consumer>
    );
}

EditionList.propTypes = {
    horizontal: PropTypes.bool,
    removeClippedSubviews: PropTypes.bool,
    initialNumToRender: PropTypes.number,
    onlyGroup: PropTypes.bool,
    edition: PropTypes.instanceOf(Edition),
    issueCallbacks: IssueItem.propTypes.issueCallbacks,
    showHighRes: PropTypes.bool,
    isSelected: PropTypes.func,
    onFavoritePress: PropTypes.func
};

EditionList.defaultProps = {
    horizontal: true,
    initialNumToRender: 5,
    removeClippedSubviews: true,
    onlyGroup: false,
    showHighRes: false
};

export default EditionList;
