import React, { useCallback, useState } from "react";
import { StyleSheet, View } from "react-native";
import PropTypes from "prop-types";
import HorizontalList from "./horizontal-list";
import VerticalList from "./vertical-list";
import { useAspectRatio } from "../../hooks/hooks";
import { THUMBNAIL_HEIGHT, THUMBNAIL_HR_THRESHOLD, updateSize } from "./list-utils";
import CssHelper from "../Css/css-methods";
import { commonStyles } from "../Css/common-styles";
import CssStyle from "../Css/css-styles";

const styles = StyleSheet.create({
    container: {
        flex: 1
    }
});

/**
 * component that choose wich kind of list render
 * evaluate also item size by checking first issue aspect ratio and parent constraint
 * @param {*} props
 * @returns
 */
function List(props) {
    const {
        horizontal,
        aspectRatioItem,
        renderItem,
        renderVerticalItem,
        renderHorizontalItem,
        cssListClasses,
        itemStyle,
        ...rest
    } = props;
    const [layout, setLayout] = useState();
    const aspectRatio = useAspectRatio(aspectRatioItem);

    const onLayout = useCallback(({ nativeEvent }) => {
        setLayout(nativeEvent.layout);
    });

    if (!aspectRatio) {
        return null;
    }

    let ToRender = null;
    if (layout) {
        const cssListContainer = CssHelper.flatten(
            [styles.container, commonStyles.margin],
            ["opt-list-container", ...cssListClasses]
        );
        //update constraints to set correct item size on window resize
        const constraints = CssStyle.updateSizeByApplingStyle(layout, cssListContainer);

        const itemFullStyle = CssHelper.flatten([{ height: THUMBNAIL_HEIGHT }, itemStyle]);
        const itemSize = updateSize({ height: itemFullStyle.height, aspectRatio, constraints });

        const ListComponent = horizontal ? HorizontalList : VerticalList;
        const listConstraintRender = horizontal ? renderHorizontalItem : renderVerticalItem;

        ToRender = (
            <ListComponent
                itemSize={itemSize}
                {...rest}
                renderItem={({ item, index }) => {
                    const itemProps = { item, index, size: itemSize, itemStyle };
                    if (typeof listConstraintRender === "function") {
                        return (
                            <View style={itemSize}>
                                {listConstraintRender({
                                    ...itemProps,
                                    showHighRes: itemSize.height > THUMBNAIL_HR_THRESHOLD
                                })}
                            </View>
                        );
                    }
                    if (typeof renderItem === "function") {
                        return <View style={itemSize}>{renderItem(itemProps)}</View>;
                    }
                    return null;
                }}
            />
        );
    }

    return (
        <View style={styles.container} onLayout={onLayout}>
            {ToRender}
        </View>
    );
}

List.propTypes = {
    renderItem: PropTypes.func,
    renderVerticalItem: PropTypes.func,
    renderHorizontalItem: PropTypes.func,
    aspectRatioItem: PropTypes.object,
    keyExtractor: PropTypes.func.isRequired,
    data: PropTypes.array.isRequired,
    cssListClasses: PropTypes.array,
    horizontal: PropTypes.bool,
    removeClippedSubviews: PropTypes.bool,
    initialNumToRender: PropTypes.number,
    itemStyle: PropTypes.oneOfType([PropTypes.array, PropTypes.object, PropTypes.number])
};

List.defaultProps = {
    horizontal: true,
    initialNumToRender: 5,
    removeClippedSubviews: true,
    cssListClasses: []
};

export default List;
