import React, { useState } from "react";
import PropTypes from "prop-types";
import { FlatList, Platform, StyleSheet, View } from "react-native";
import { commonStyles } from "../Css/common-styles";
import { Css } from "../Css/css";
import CssHelper from "../Css/css-methods";

const styles = StyleSheet.create({
    container: {
        overflow: "hidden"
    },
    list: {
        ...Platform.select({
            web: {
                overflow: "auto",
                height: "100%"
            }
        })
    }
});

/**
 * render issue list in vertical mode,
 * evaluate also the number of columns
 * @param {*} props
 * @returns
 */
function VerticalList(props) {
    const {
        data,
        itemSize: defaultItemSize,
        containerStyle,
        contentContainerStyle,
        renderItem,
        gridMode,
        numColumns: defaultNumColumns,
        ...rest
    } = props;

    const [state, setState] = useState({});

    const onLayout = ({ nativeEvent }) => {
        const { layout } = nativeEvent;
        const itemSize = { ...defaultItemSize };
        if (!itemSize.width) {
            itemSize.width = Math.floor(layout.width / defaultNumColumns || 1);
        }
        let nCols = defaultNumColumns || Math.floor(layout.width / itemSize.width) || 1;

        /**
         * try to fill better the grid
         */
        const availableRightSpace = layout.width - itemSize.width * nCols;
        const aspectRatio = itemSize.width / itemSize.height;
        if (availableRightSpace < itemSize.width * 0.5) {
            itemSize.width += Math.floor(availableRightSpace / nCols);
            itemSize.height = itemSize.width / aspectRatio;
        } else if (availableRightSpace > itemSize.width * 0.75) {
            nCols += 1;
            itemSize.width = Math.floor(layout.width / nCols);
            itemSize.height = itemSize.width / aspectRatio;
        }

        setState({
            layout,
            numColumns: nCols,
            itemSize
        });
    };

    const { layout, numColumns, itemSize } = state;

    return (
        <Css classes={["opt-issues-container", "opt-issues-container-vertical"]}>
            <View
                style={[commonStyles.container, styles.container, containerStyle]}
                onLayout={onLayout}
                collapsable={false}>
                {layout ? (
                    <Css classes={["opt-issues-list", "opt-issues-list-vertical"]}>
                        <FlatList
                            key={`edition_list_cols_${numColumns}`}
                            data={data}
                            contentContainerStyle={CssHelper.flatten(
                                [styles.list, contentContainerStyle],
                                ["opt-issues-list-container", "opt-issues-list-container-vertical"]
                            )}
                            style={{ overflow: "visible" }}
                            renderItem={({ item, index }) => {
                                const style = [itemSize];
                                //if gridMode compute the margin of the item automatically
                                if (gridMode && numColumns > 1) {
                                    const margin = (layout.width - itemSize.width * numColumns) / (numColumns - 1) / 2;
                                    if (index % numColumns === 0) {
                                        style.push({ marginRight: margin });
                                    } else if ((index + 1) % numColumns === 0) {
                                        style.push({ marginLeft: margin });
                                    } else {
                                        style.push({ marginHorizontal: margin });
                                    }
                                }
                                const itemProps = { item, index, style, size: itemSize };
                                return renderItem(itemProps);
                            }}
                            {...rest}
                            numColumns={numColumns}
                        />
                    </Css>
                ) : null}
            </View>
        </Css>
    );
}

VerticalList.propTypes = {
    data: PropTypes.array,
    itemSize: PropTypes.shape({
        width: PropTypes.number,
        height: PropTypes.number
    }),
    numColumns: PropTypes.number,
    renderItem: PropTypes.func,
    keyExtractor: PropTypes.func,
    containerStyle: PropTypes.oneOfType([PropTypes.number, PropTypes.object, PropTypes.array]),
    contentContainerStyle: PropTypes.oneOfType([PropTypes.number, PropTypes.object, PropTypes.array]),
    gridMode: PropTypes.bool
};

export default VerticalList;
