import React, { useCallback, useEffect, useRef } from "react";
import RNWebView, { WebViewProps } from "react-native-webview";
import Logger from "../../utils/log";
import { segmentURL } from "../../utils/strings";
import { EventRegister } from "../../events/manager";
import Events from "../../events/events";
import Message from "../../models/message";
import { Linking } from "react-native";

function TaWebView(props) {
    const ref = useRef(null);

    const jsInjection = ({ js } = {}) => {
        if (ref && ref.current && js) {
            ref.current.injectJavaScript(/* javascript */ `
                try{
                    ${js}
                } catch(e){
                    console.log(e.message);
                }`);
        }
    };

    const shouldStartLoad = useCallback(({ url, navigationType }) => {
        if (url.startsWith("local://")) {
            const uri = segmentURL(url);
            Logger.debug(`domain: ${uri.getDomain()}`);
            EventRegister.emit(Events.WEBVIEW_ON_LOCAL, { uri, jsInjection });
            return false;
        }
        return true;
    }, []);

    const onMessage = useCallback(({ nativeEvent }) => {
        EventRegister.emit(Events.WEBVIEW_ON_MESSAGE, {
            message: new Message(nativeEvent),
            jsInjection
        });
    }, []);

    //iOS only
    const onFileDownload = useCallback(({ nativeEvent: { downloadUrl } }) => {
        Linking.openURL(downloadUrl);
    }, []);

    /**
     * all active webviews are listening to js injection, so pay attetion before trigger this event
     */
    useEffect(() => {
        const jsInjectId = EventRegister.on(Events.JS_INJECTION, jsInjection);
        return () => {
            EventRegister.rm(jsInjectId);
        };
    }, []);

    return (
        <RNWebView
            ref={ref}
            onShouldStartLoadWithRequest={shouldStartLoad}
            originWhitelist={["http://", "https://", "xhttp://", "xhttps://", "local://", "file://"]}
            onMessage={onMessage}
            mixedContentMode="always"
            allowFileAccess
            allowUniversalAccessFromFileURLs
            thirdPartyCookiesEnabled
            domStorageEnabled
            javaScriptEnabled
            bounces={false}
            cacheEnabled
            startInLoadingState
            androidForceHasOverlappingRendering={false}
            messagingEnabled
            onFileDownload={onFileDownload}
            {...props}
        />
    );
}

TaWebView.propTypes = {
    ...WebViewProps
};

export default TaWebView;
