import PaperSetupHelper from "../helpers/paper-setup-helper";
import { TABS } from "../interfaces/navigations";
import { getPSetup } from "../utils/api";
import JSUtils from "../utils/js-utils";
import Logger from "../utils/log";
import Storage from "../utils/storage";

const { keys } = PaperSetupHelper.SP;

export default class PaperSetupConfig {
    #_json: Object;
    #_storageKey: String;
    #_pSetup: String;

    paper: String;
    machine: String;
    defaultSelectedEditions: String;
    language: Number;
    pSetupFileTime: Number = -1;
    maxEditionsNotGrouped: Number = 1;
    maxEditionsGroupLabel: String;
    customLanguagePackFileTime: Number = -1;
    showThemesCustomization: Boolean = false;
    disableRnStyles: Boolean = true;
    rnStyles: Object = {};
    optionsPageTabPath: String = "";
    tabs: Array<> = [];
    orderEditions: Array<String> = [];
    loggedInTabs: Array<String> = [];
    sortingTabs: Array<String> = [];
    newsstandTabs: Array<String> = [];

    editions: Array = [];

    xpaperUniversalLinks: Object = {};

    constructor(pSetup) {
        this.#_pSetup = pSetup;
        this.#_storageKey = `opt-${pSetup}`;
    }

    async download(pSetupOverride = {}) {
        Logger.debug(`Loading pSetup ${this.#_pSetup}`);
        let json;
        try {
            json = await getPSetup({
                pSetup: this.#_pSetup
            });
        } catch (e) {
            Logger.warn(e.message);
        }
        //if something fails try to load from storage
        if (json) {
            Logger.debug("Clear previous stored pSetup values");
            this.#_json = { ...json, ...pSetupOverride };
            Storage.clearAll(this.#_storageKey).then(() => Storage.multiSet(this.#_storageKey, this.#_json));
        } else {
            this.#_json = await Storage.multiGet(this.#_storageKey);
        }
    }

    async init() {
        Logger.debug(`Initializing pSetup ${this.#_pSetup}`);
        try {
            this.paper = await this.#_getValue(keys.paper);
            this.machine = await this.#_getValue(keys.machine);
            this.defaultSelectedEditions = await this.#_getValue(keys.default_selected_editions);
            this.customLanguagePackFileTime = parseInt(
                (await this.#_getValue(keys.custom_language_pack_file_time)) || 1,
                10
            );
            this.pSetupFileTime = parseInt((await this.#_getValue(keys.file_time)) || -1, 10);
            this.language = await this.#_getValue(keys.language);
            this.disableRnStyles = (await this.#_getValue(keys.disable_rn_styles)) === "1";
            this.showThemesCustomization = (await this.#_getValue(keys.show_themes_customization)) === "1";
            this.logoUrl = await this.#_getValue(keys.logo_url);
            this.optionsPageTabPath = await this.#_getValue(keys.options_page_tab_path);
            this.maxEditionsNotGrouped = parseInt((await this.#_getValue(keys.max_editions_not_grouped)) || "1", 10);
            this.maxEditionsGroupLabel = await this.#_getValue(keys.max_editions_group_label);
            try {
                this.tabs = JSON.parse(await this.#_getValue(keys.tabs));
            } catch (e) {
                Logger.warn(e.message);
                this.tabs = [];
            }

            try {
                const tempValue = await this.#_getValue(keys.order_editions);
                if (tempValue) {
                    this.orderEditions = tempValue.split(",");
                }
            } catch (e) {
                this.orderEditions = [];
            }
            try {
                this.loggedInTabs = JSON.parse(await this.#_getValue(keys.logged_in_tabs));
            } catch (e) {
                Logger.debug(e.message);
            }
            try {
                this.sortingTabs = JSON.parse(await this.#_getValue(keys.sorting_tabs));
            } catch (e) {
                Logger.debug(e.message);
            }
            try {
                this.newsstandTabs = JSON.parse(await this.#_getValue(keys.newsstand_tabs));
            } catch (e) {
                Logger.debug(e.message);
                this.newsstandTabs = [
                    //TABS.LATEST_EDITIONS,
                    TABS.CUSTOM_EDITIONS,
                    TABS.OTHER_EDITIONS,
                    TABS.ARCHIVE_EDITIONS
                ];
            }

            const rnStyleRaw = await this.#_getValue(keys.rn_styles);
            if (rnStyleRaw) {
                try {
                    this.rnStyles = JSON.parse(rnStyleRaw);
                } catch (e) {
                    Logger.warn("failed to parse rn styles from action message");
                }
            } else {
                Logger.debug("No custom rn styles provided with action message");
            }

            try {
                this.editions = await this.#_getValue(keys.editions);
            } catch (e) {
                Logger.warn("failed to parse editions from action message");
            }

            const xpaperUniversalLinksRaw = await this.#_getValue(keys.xpaper_universal_links);
            if (rnStyleRaw) {
                try {
                    this.xpaperUniversalLinks = JSON.parse(xpaperUniversalLinksRaw);
                } catch (e) {
                    Logger.warn("failed to parse rn styles from action message");
                }
            }

            return true;
        } catch (e) {
            Logger.warn(e.message);
        }
        return false;
    }

    setEditions(editions) {
        this.editions = editions;
        this.#_setValue(keys.editions, JSON.stringify(editions));
    }

    async #_setValue(key, value) {
        this.#_json[key] = value;
        return PaperSetupHelper.setItem(this.#_storageKey, key, value);
    }

    async #_getValue(key) {
        if (JSUtils.hasProperty(this.#_json, key)) {
            let value = this.#_json[key];
            if (value === null || value === undefined) {
                Logger.debug(`${key}: ${value}`);
                value = await PaperSetupHelper.getItem(this.#_storageKey, key);
            }
            return value;
        }
        return "";
    }
}
