import LiveConfig from "../data/live-config";
import Edition from "../models/edition";
import Issue from "../models/issue";
import Publication from "../models/publication";
import Special from "../models/special";
import DateUtils from "../utils/date";
import { decodeHTML } from "../utils/strings";

class EditionsHelper {
    static #_getEditionInfoFromName(editionName = "") {
        const splitted = editionName.split("---");
        return {
            name: splitted[0],
            lazy: splitted[1] !== undefined
        };
    }

    static #_getIssue(editionName, key, data): Issue {
        const {
            filename,
            closed,
            public: isPublic,
            lastClosed,
            dayOfWeek,
            dayOfWeekComplete,
            pSetup,
            dateLabelAccessibility,
            dateLabel,
            pathEdi,
            siteDomain,
            siteDomainCDN,
            machine
        } = data;
        const mIssue = new Issue();
        mIssue.setIssue(key);
        mIssue.setEdition(editionName);
        mIssue.setFilename(filename);
        mIssue.setLastClosed(lastClosed);
        mIssue.setDayOfWeek(dayOfWeek);
        mIssue.setDayOfWeekComplete(decodeHTML(dayOfWeekComplete));
        mIssue.setPSetup(pSetup);
        mIssue.setDateLabelAccessibility(decodeHTML(dateLabelAccessibility));
        mIssue.setDateLabel(decodeHTML(dateLabel));
        mIssue.setPathEdition(pathEdi);
        mIssue.setSiteDomain(siteDomain);
        mIssue.setSiteDomainCDN(siteDomainCDN);
        mIssue.setMachine(machine);
        mIssue.setPublic(isPublic);
        //seems that closed is in only if is 0, in that case missing lastClosed
        mIssue.setClosed(closed);
        return mIssue;
    }

    static #_getIssues(editionName, pics = {}): Array<Issue> {
        const mIssues = [];
        const editionIssues = pics[editionName];
        if (editionIssues) {
            const info = EditionsHelper.#_getEditionInfoFromName(editionName);
            Object.keys(editionIssues).forEach((issue) => {
                mIssues.push(EditionsHelper.#_getIssue(info.name, issue, editionIssues[issue]));
            });
        }
        return mIssues;
    }

    static #_getEditions(edition = {}, pics = {}, parent = "", isParentSecondaryEdition): Array<Edition> {
        const mEditions = [];
        Object.keys(edition).forEach((name) => {
            const {
                label_edition = name,
                subtitle_edition,
                group,
                force_position = false,
                is_secondary_edition = isParentSecondaryEdition,
                is_shortcut_edition = false,
                pSetup,
                icon
            } = edition[name];
            const info = EditionsHelper.#_getEditionInfoFromName(name);
            const mEdition = new Edition();
            mEdition.setName(info.name);
            mEdition.setPSetup(pSetup);
            mEdition.setIsMain(pSetup === LiveConfig.getMainPSetupName());
            mEdition.setLabel(decodeHTML(label_edition));
            mEdition.setSubLabel(subtitle_edition);
            mEdition.setIsSecondary(is_secondary_edition);
            mEdition.setIsShortcut(is_shortcut_edition);
            mEdition.setForcePosition(force_position);
            mEdition.setGroup(EditionsHelper.#_getEditions(group, pics, name, is_secondary_edition));
            mEdition.setParent(parent);
            mEdition.setIcon(icon);
            mEdition.setLazy(info.lazy);
            //reverse to have most recent issue in the first position
            mEdition.setIssues(EditionsHelper.#_getIssues(name, pics).reverse());
            mEditions.push(mEdition);
        });
        return mEditions.sort((e1: Edition, e2: Edition) => {
            if (e1.isForcePosition() || e2.isForcePosition()) {
                return 0;
            }
            if (e1.getName() < e2.getName()) {
                return -1;
            }
            if (e1.getName() > e2.getName()) {
                return 1;
            }
            return 0;
        });
    }

    static getRouteName(text) {
        //return text.replace(/ /g, "_");
        return text;
    }

    /**
     * if closesIssue is empty return the first element in array otherwise
     * return issue equal to closes issue or first less issue found in array
     * @param {*} issues
     * @param {*} closestIssue
     * @returns
     */
    static getClosestIssue(issues: Array<Issue> = [], closestIssue: String = "") {
        return (
            issues.find((issue) => issue.getIssue() === closestIssue || issue.getIssue() < closestIssue) || issues[0]
        );
    }

    static getIssuesFromPublications(
        publications: Array<Publication> = [],
        mainEdition,
        { forceShortcut = false, closestIssue = "" }
    ): Array<Issue> {
        const issues = [];
        publications.forEach((publication) => {
            if (publication.getName() !== mainEdition) {
                let issuesToAdd = [];
                if (publication.isShortcut() || forceShortcut) {
                    if (publication instanceof Publication) {
                        const subEditions = publication.getGroup();
                        if (subEditions && subEditions.length > 0) {
                            issuesToAdd.push(
                                ...EditionsHelper.getIssuesFromPublications(subEditions, mainEdition, {
                                    forceShortcut: true,
                                    closestIssue
                                })
                            );
                        } else {
                            issuesToAdd.push(EditionsHelper.getClosestIssue(publication.getIssues(), closestIssue));
                        }
                    } else {
                        issuesToAdd.push(EditionsHelper.getClosestIssue(publication.getIssues(), closestIssue));
                    }
                } else {
                    issuesToAdd = publication.getIssues();
                }

                issuesToAdd.forEach((issueToAdd) => {
                    if (
                        !issues.find(
                            (issue) =>
                                // eslint-disable-next-line implicit-arrow-linebreak
                                issue.getEdition() === issueToAdd.getEdition() &&
                                issue.getIssue() === issueToAdd.getIssue()
                        )
                    ) {
                        issues.push(issueToAdd);
                    }
                });
            }
        });
        return issues;
    }

    static parseEditions({ edition, pics } = {}): Array<Edition> {
        return EditionsHelper.#_getEditions(edition, pics, "", false);
    }

    static parseSpecials(specials = {}): Array<Special> {
        const mSpecials = [];
        Object.keys(specials).forEach((name) => {
            const mSpecial = new Special();
            mSpecial.setName(name);
            mSpecial.setLazy(name.match(/.+---[0-9]+/) !== null);
            mSpecial.setIssues(EditionsHelper.#_getIssues(name, specials));
            mSpecials.push(mSpecial);
        });
        return mSpecials;
    }

    static getShortuctEditions(editions: Array<Edition>): Array<Edition> {
        return editions.filter((edition) => edition.isShortcut());
    }

    static getSecondaryEditions(editions: Array<Edition>): Array<Edition> {
        return editions.reduce((result, edition: Edition) => {
            if (!edition.isSecondary()) {
                result.push(...EditionsHelper.getSecondaryEditions(edition.getGroup()));
            } else {
                result.push(edition);
            }
            return result;
        }, []);
    }

    static getPublicationFromIssue(editions: Array<Publication> = [], issue: Issue) {
        let publication;
        if (issue) {
            publication = editions.find((edition) => edition.getName() === issue.getEdition());
            if (!publication) {
                editions.forEach((edition) => {
                    const group = edition.getGroup();
                    if (group && group.length > 0) {
                        publication = EditionsHelper.getPublicationFromIssue(group, issue);
                        if (publication) {
                            publication.setIsShortcut(edition.isShortcut());
                        }
                    }
                });
            }
        }
        return publication;
    }

    static getEditionByName(editions: Array<Edition> = [], name: String): Edition {
        for (let i = 0; i < editions.length && name; i++) {
            if (editions[i].getName() === name) {
                return editions[i];
            }
            const found = EditionsHelper.getEditionByName(editions[i].getGroup(), name);
            if (found) {
                return found;
            }
        }
        return undefined;
    }

    static getValidIssue(issues: Array<Issue>) {
        for (let j = 0; j < issues.length; j++) {
            const issue = issues[j];
            if (issue) {
                return issue;
            }
        }
        return undefined;
    }

    static populateIssueOnEditions(editions: Array<Edition> = [], pics = {}) {
        for (let i = 0; i < editions.length; i++) {
            const issues = EditionsHelper.#_getIssues(editions[i].getName(), pics).reverse();
            if (issues.length) {
                editions[i].setIssues(issues);
                //disable lazy, issues are loaded
                editions[i].setLazy(false);
            }
            if (editions[i].getGroup().length) {
                EditionsHelper.populateIssueOnEditions(editions[i].getGroup(), pics);
            }
        }
    }

    static getFirstValidIssue(editions: Array<Edition> = []) {
        let issue;
        for (let i = 0; i < editions.length && !issue; i++) {
            const edition = editions[i];
            if (edition) {
                if (edition.getIssues()) {
                    issue = EditionsHelper.getValidIssue(edition.getIssues());
                }
                if (!issue && edition.getGroup()) {
                    issue = EditionsHelper.getFirstValidIssue(edition.getGroup());
                }
            }
        }
        return issue;
    }

    static getPreviousLevel(edition: Edition, child: Edition, level = 0) {
        if (edition.hasChild(child)) {
            return {
                parent: level ? edition : undefined,
                items: edition.getGroup()
            };
        }
        const children = edition.getGroup();
        for (let i = 0; i < children.length; i++) {
            const found = this.getPreviousLevel(children[i], child, level + 1);
            if (found) {
                return found;
            }
        }
        return undefined;
    }

    static getIssueLabel(issue: Issue) {
        let label = "";
        if (issue) {
            label = issue.getDayOfWeekComplete();
            if (issue.getIssue() === DateUtils.getYesterdayString("YYYYMMDD")) {
                label = LiveConfig.getLocale().strings("str_yesterday");
            }
        }
        return label;
    }

    static getMainEdition(edition: Edition) {
        try {
            if (edition) {
                return edition.split(" _ ")[0];
            }
        } catch (e) {
            //ignore
        }
        return "";
    }

    static getAllOtherIssuesForDate(editions, issueDate) {
        const editionIssues = [];
        editions.forEach((edition) => {
            if (!edition.isMain()) {
                if (edition.getGroup().length > 0) {
                    editionIssues.push(...EditionsHelper.getAllOtherIssuesForDate(edition.getGroup(), issueDate));
                } else {
                    editionIssues.push(...edition.getIssues().filter((issue) => issue.getIssue() === issueDate));
                }
            }
        });

        return editionIssues;
    }
}

export default EditionsHelper;
