import LiveConfig from "../data/live-config";
import { TABS } from "../interfaces/navigations";
import Edition from "../models/edition";
import Route from "../models/route";
import RouteEdition from "../models/route-edition";
import RouteEditionsDate from "../models/route-editions-date";
import RouteTabs from "../models/route-tabs";
import RouteUrl from "../models/route-url";
import Network from "../utils/network";
import { addParamsToURL } from "../utils/strings";
import EditionsHelper from "./editions-helper";

class RoutesHelper {
    /**
     * TODO check also psetup values
     * @param {*} routes
     * @param {*} routeName
     * @returns Route
     */
    static getInitialRoute(routes: Array<Route> = [], routeName: String): Route {
        if (routes.length) {
            for (let i = 0; i < routes.length; i++) {
                if (routes[i] instanceof RouteEdition) {
                    const route: RouteEdition = routes[i];
                    if (route.getEdition().getName() === routeName) {
                        return route;
                    }
                } else if (routes[i] instanceof RouteUrl) {
                    const route: RouteUrl = routes[i];
                    if (route.getRoute() === routeName) {
                        return route;
                    }
                }

                const found = RoutesHelper.getInitialRoute(routes[i].getRoutes(), routeName);
                if (found) {
                    //expand parent of selected route to see selected items layout
                    routes[i].setCollapsed(false);
                    return found;
                }
            }
        }
        return undefined;
    }

    static getUrl(php, customUrl, customUrlTarget) {
        let newUrl = customUrl ? customUrl : "";
        let newTarget = customUrlTarget ? "_blank" : "";
        const params = { ...LiveConfig.urlParams(), pSetup: LiveConfig.getPSetupName(), native: 1 };
        if (php === "ipad_login.php") {
            params.fromOpt = 1;
        }
        if (newUrl) {
            //relative path
            if (newUrl[0] === "/") {
                newUrl = `${LiveConfig.getServer()}${newUrl}`;
            }
            newUrl = Network.addDefaultProtocol(newUrl);
            return { url: newUrl, target: newTarget };
        }
        //custom target disabled by default on HTML5 for puzzles
        const url = addParamsToURL(
            `${LiveConfig.getServer()}${LiveConfig.getPSetup().optionsPageTabPath}${php}`,
            params
        );
        //if the site is HTTPS but the custom link not force to open in a new tab
        if (Network.isHTTPS && url.indexOf("http:") > -1 && !newTarget) {
            newTarget = "_blank";
        }
        return { url, target: newTarget };
    }

    static getMainRoute(editions: Array<Edition>, otherRoutes: Array<Route> = []): RouteTabs {
        const route = new RouteTabs(LiveConfig.getLocale().strings("str_newsstand"));
        route.setRoute(TABS.NEWSSTAND);
        route.setRoutes([...RoutesHelper.getEditionRoutes(editions), ...otherRoutes]);
        return route;
    }

    static getUrlRoutes(urls: Object, level = 0, parent = ""): Array<RouteUrl> {
        const drawerRoutes = [];
        const tabsRoutes = [];
        Object.keys(urls).forEach((key) => {
            const { php, label, customUrl, customUrlTarget, showIfIsLoggedIn, icon, isTabElement } = urls[key];
            const { url, target } = RoutesHelper.getUrl(php, customUrl, customUrlTarget);
            const mRoute = new RouteUrl();
            mRoute.setRoute(key);
            mRoute.setLabel(label || key);
            mRoute.setUrl(url);
            mRoute.setTarget(target);
            mRoute.setIcon(icon);
            mRoute.setLevel(level);
            mRoute.setParent(parent);
            mRoute.setVisibleIfLoggedIn(showIfIsLoggedIn);
            mRoute.setCollapsed(true);
            if (isTabElement) {
                tabsRoutes.push(mRoute);
            } else {
                drawerRoutes.push(mRoute);
            }
        });
        return { drawerRoutes, tabsRoutes };
    }

    static makeEditionModel(group: Array<Edition>, name: String) {
        const mEdition = new Edition();
        mEdition.setGroup(group);
        mEdition.setName(name);
        return mEdition;
    }

    static getEditionRoute(config = {}) {
        const { mEdition, routeName, label, fixed = true, index = 0 } = config;
        const mRoute = new RouteEditionsDate();
        mRoute.setRoute(routeName);
        mRoute.setEdition(mEdition);
        mRoute.setLabel(label);
        mRoute.setIssueDayIndex(index);
        mRoute.setFixed(fixed);
        return mRoute;
    }

    static getAllOtherIssuesForDate(editions, issueDate) {
        const editionIssues = [];
        editions.forEach((edition) => {
            if (!edition.isMain() && !edition.isSecondary()) {
                if (edition.getGroup().length > 1) {
                    editionIssues.push(...RoutesHelper.getAllOtherIssuesForDate(edition.getGroup(), issueDate));
                } else {
                    editionIssues.push(...edition.getIssues().filter((issue) => issue.getIssue() === issueDate));
                }
            }
        });

        return editionIssues;
    }

    static getEditionRoutes(editions: Array<Edition>): Array<RouteEdition> {
        const { newsstandTabs, maxEditionsGroupLabel, maxEditionsNotGrouped } = LiveConfig.getPSetup();

        const routes = [];

        const mainEditions = editions.filter((edition) => edition.isMain());
        const mainEdition = mainEditions[0];

        newsstandTabs.forEach((tab: String) => {
            if (tab === TABS.LATEST_EDITIONS) {
                routes.push(
                    RoutesHelper.getEditionRoute({
                        mEdition: RoutesHelper.makeEditionModel(editions, "latest_editions_group"),
                        routeName: TABS.LATEST_EDITIONS,
                        label: LiveConfig.getLocale().strings("str_latest")
                    })
                );
            } else if (tab === TABS.CUSTOM_EDITIONS) {
                if (mainEdition) {
                    routes.push(
                        RoutesHelper.getEditionRoute({
                            mEdition: RoutesHelper.makeEditionModel(editions, "custom_editions_group"),
                            routeName: TABS.CUSTOM_EDITIONS,
                            label: LiveConfig.getLocale().strings("str_today"),
                            index: 0,
                            fixed: false
                        })
                    );
                }
            } else if (tab === TABS.PENULTIMATE_EDITIONS) {
                if (mainEdition) {
                    const dayIndex = 1;
                    const dayIssue = mainEdition.getIssues()[dayIndex];
                    //add only if exists something
                    if (dayIssue) {
                        routes.push(
                            RoutesHelper.getEditionRoute({
                                mEdition: RoutesHelper.makeEditionModel(editions, "custom_editions_group"),
                                routeName: TABS.PENULTIMATE_EDITIONS,
                                label: EditionsHelper.getIssueLabel(dayIssue),
                                index: dayIndex
                            })
                        );
                    }
                }
            } else if (tab === TABS.OTHER_EDITIONS) {
                if (maxEditionsNotGrouped > -1 && maxEditionsNotGrouped < editions.length) {
                    const mRoute = new RouteEdition();
                    mRoute.setRoute(TABS.OTHER_EDITIONS);
                    mRoute.setEdition(RoutesHelper.makeEditionModel(editions, "max_editions_grouped"));
                    mRoute.setLabel(maxEditionsGroupLabel || LiveConfig.getLocale().strings("grouped_editions"));
                    routes.push(mRoute);
                } else {
                    editions.forEach((edition) => {
                        if (!edition.isMain()) {
                            const mRoute = new RouteEdition();
                            mRoute.setRoute(EditionsHelper.getRouteName(edition.getName()));
                            mRoute.setEdition(edition);
                            mRoute.setLabel(edition.getLabel());
                            routes.push(mRoute);
                        }
                    });
                }
            } else if (tab === TABS.ARCHIVE_EDITIONS) {
                const archiveEditions = [];
                //TODO: handle multi edition
                const issues = mainEdition.getIssues();
                for (let i = 0; i < issues.length; i++) {
                    const editionIssues = [];
                    editionIssues.push(
                        issues[i],
                        ...RoutesHelper.getAllOtherIssuesForDate(editions, issues[i].getIssue())
                    );

                    const edition = new Edition();
                    edition.setIssues(editionIssues);
                    edition.setLabel(issues[i].getDateLabel());
                    edition.setName(issues[i].getDateLabel());

                    archiveEditions.push(edition);
                }

                const mEdition = new Edition();
                mEdition.setGroup(archiveEditions);
                mEdition.setName("archive_editions_group");

                const archive = new RouteEdition();
                archive.setEdition(mEdition);
                archive.setRoute(TABS.ARCHIVE_EDITIONS);
                archive.setLabel(LiveConfig.getLocale().strings("str_archive_editions"));
                routes.push(archive);
            }
        });

        return routes;
    }
}

export default RoutesHelper;
