import AsyncStorage from "@react-native-async-storage/async-storage";
import Queue from "promise-queue";
import Logger from "./log";

export default class Storage {
    /*
     * the following methods store and read information to and from shared preferences/user defaults
     * DO NOT REMOVE QUEUE
     */

    static #_queue = new Queue(1, Infinity);

    /*
     * the following methods store and read information to and from react native AsyncStorage
     * default method
     */

    static #_appendNameToKey(name, key) {
        return `@${name}:${key}`;
    }

    static async setItem(name, key, value) {
        try {
            const namedKey = Storage.#_appendNameToKey(name, key);
            await AsyncStorage.setItem(namedKey, value);
        } catch (error) {
            Logger.warn(`Failed to setItem ${key} => ${value} due to error ${error.message}`);
        }
    }

    static async multiSet(name, data) {
        try {
            const toSet = Object.keys(data).map((key) => {
                const namedKey = Storage.#_appendNameToKey(name, key);
                return [namedKey, Array.isArray(data[key]) ? JSON.stringify(data[key]) : data[key]];
            });
            await AsyncStorage.multiSet(toSet);
        } catch (error) {
            Logger.warn(`Failed to arrays ${error.message}`);
        }
    }

    static async multiGet(name, editCallback) {
        let result = {};
        try {
            const keys = (await AsyncStorage.getAllKeys()).filter((key) => key.includes(`@${name}:`));
            result = (await AsyncStorage.multiGet(keys)).reduce((multiGet, item) => {
                multiGet[item[0].replace(`@${name}:`, "")] = editCallback ? editCallback(item[1]) : item[1];
                return multiGet;
            }, {});
        } catch (error) {
            Logger.warn(`Failed to get all ${error.message}`);
        }
        return result;
    }

    static async getItem(name, key) {
        let result = "";
        try {
            if (key) {
                const namedKey = Storage.#_appendNameToKey(name, key);
                result = await AsyncStorage.getItem(namedKey);
            }
        } catch (error) {
            Logger.warn(`Failed to getItem ${key} due to error ${error.message}`);
        }
        return result;
    }

    static async clear(name, key) {
        try {
            await AsyncStorage.removeItem(Storage.#_appendNameToKey(name, key));
        } catch (e) {
            Logger.warn(`Failed to delete ${key} > ${e.message}`);
        }
    }

    static async clearAll(name) {
        try {
            const keys = (await AsyncStorage.getAllKeys()).filter((key) => key.includes(`@${name}:`));
            await AsyncStorage.multiRemove(keys);
        } catch (e) {
            Logger.warn(`Failed to delete all from ${name} > ${e.message}`);
        }
    }

    static async clearAllByKeys(name, keys) {
        try {
            const namedkeys = keys.map((key) => `@${name}:${key}`);
            await AsyncStorage.multiRemove(namedkeys);
        } catch (e) {
            Logger.warn(`Failed to delete all from ${name} > ${e.message}`);
        }
    }

    static async clearAllByRegExp(name) {
        try {
            const keys = (await AsyncStorage.getAllKeys()).filter((key) => !!key.match(`@${name}:`));
            await AsyncStorage.multiRemove(keys);
        } catch (e) {
            Logger.warn(`Failed to delete all from ${name} > ${e.message}`);
        }
    }
}
