import "react-native-gesture-handler";
import "./app.css";

/**
 * @format
 */
import React from "react";
import * as ReactDOM from "react-dom";
import Root from "./src";
import JSUtils from "./src/utils/js-utils";
import { EventRegister } from "./src/events/manager";
import Events from "./src/events/events";

export const _GET = (location) => {
    const url = new URL(location);
    const keys = Array.from(url.searchParams.keys());
    return keys.reduce((result, key) => {
        result[key] = url.searchParams.get(key);
        return result;
    }, {});
};

export const emit = (data = {}) => {
    const message = JSON.stringify(data);
    if (window.ReactNativeWebView && window.ReactNativeWebView.postMessage) {
        window.ReactNativeWebView.postMessage(message);
    } else {
        window.postMessage(message, window.origin);
    }
};

export const iframeInjection = (js) => {
    try {
        EventRegister.emit(Events.IFRAME_JS_INJECTION, { js: decodeURIComponent(js) });
    } catch (e) {
        console.warn(e.message);
    }
};

const waiting = {};
const AWAITING_TIMEOUT = __DEV__ ? 100 : 5000;

const parsePayload = (payload) => {
    if (payload === "true") {
        return true;
    }
    if (payload === "false") {
        return false;
    }
    const result = parseFloat(payload);
    if (!Number.isNaN(result)) {
        return result;
    }
    try {
        return JSON.parse(payload);
    } catch (e) {
        throw e;
    }
};

/**
 * resolve or reject the promise attacched to this key
 * @param {*} key contained inside postMessage payload
 * @param {string} method reject|resolve
 * @param {string} payload the string, if resolve will be parsed with JSON.parse otherwise is the error message
 */
export const awaitPostMessage = (key, method, payload) => {
    try {
        if (method === "resolve") {
            waiting[key].resolve(parsePayload(payload));
        } else {
            waiting[key].reject(new Error(payload));
        }
    } catch (e) {
        if (waiting[key] && waiting[key].reject) {
            waiting[key].reject(e);
        }
    }
    delete waiting[key];
};

/**
 * call this method to post a message and awaiting a response from native code
 * each message has a unique key, used to identified the promise and resolve it
 * to resolve the promise from native code you need to inject js awaitPostMessage,
 * see the method below
 * @param {*} data
 * @returns Promise
 */
export const asyncPostMessage = async (data) => {
    return new Promise((resolve, reject) => {
        try {
            const key = JSUtils.generateId();
            waiting[key] = { resolve, reject };
            emit({ asyncKey: key, ...data });
            setTimeout(() => {
                awaitPostMessage(key, "reject", `${key} went in timeout`);
            }, AWAITING_TIMEOUT);
        } catch (e) {
            console.warn(e.message);
            reject(e);
        }
    });
};

export const render = (nodeId, props = {}) => {
    const element = document.getElementById(nodeId);
    if (element) {
        element.style.display = "flex";
        element.style.height = "100%";
        ReactDOM.render(<Root {...props} />, element);
        return () => {
            if (element) {
                ReactDOM.unmountComponentAtNode(element);
            }
        };
    }
    throw new Error(`element ${nodeId} must be a valid node`);
};

export * from "./src";
