import React, { useEffect, useRef, useState } from "react";
import PropTypes from "prop-types";
import { View, TouchableOpacity, StyleSheet, Animated } from "react-native";
import { Css } from "../Css/css";
import CollapseAccordion from "./collapse-accordion";
import { colors } from "../../utils/colors";
import { CARD_RADIUS } from "../Css/common-styles";
import LiveConfig from "../../data/live-config";

const HEIGHT = 50;

function collapseItemHeight() {
    return HEIGHT;
}

const styles = StyleSheet.create({
    line: {
        height: HEIGHT,
        marginEnd: 10,
        paddingLeft: 5,
        alignItems: "center",
        flexDirection: "row"
    },
    lineActive: {
        backgroundColor: colors.grey
    },
    lineLeft: {
        flex: 1,
        justifyContent: "center"
    },
    lineRight: {
        justifyContent: "center"
    },
    accordionContainer: {
        padding: 5
    }
});

/**
 * render drawer item with animation on collapse function
 * @param {*} props
 * @returns
 */
function CollapseLine(props) {
    const {
        line,
        lineActive,
        lineHeight,
        lineChildren,
        lineCollapsed,
        lineLevel,
        lineToggle,
        onToggle,
        css,
        renderLineLeft,
        renderLineAccordion,
        renderLineChild
    } = props;
    const {
        key,
        classesContainer = [],
        classesIconContainer = [],
        classesIcon = [],
        classesChildrenContainer = []
    } = css;

    const animatedHeight = useRef(new Animated.Value(lineHeight)).current;
    const currentHeight = useRef(lineHeight);
    const [collapsed, setCollapsed] = useState(lineCollapsed);

    useEffect(() => {
        setCollapsed(lineCollapsed);
    }, [lineCollapsed]);

    const hasChildren = lineChildren.length > 0;
    let animation;

    const animate = (toValue, callback) => {
        if (currentHeight.current !== toValue) {
            currentHeight.current = toValue;
            if (animation) {
                animation.stop();
            }
            //useNativeDriver must be false, is not supported by rn-web
            animation = Animated.timing(animatedHeight, {
                duration: 300,
                toValue,
                useNativeDriver: false
            }).start(({ finished }) => {
                if (finished) {
                    animation = undefined;
                    if (typeof callback === "function") {
                        callback();
                    }
                }
            });
        }
    };

    /**
     * this callback is important to update parent height when children change its collapse state and its height
     */
    const onChildrenLayout = ({ nativeEvent }) => {
        const startChildrenLayout = lineChildren.length * lineHeight;
        const toValue = collapsed
            ? lineHeight
            : lineHeight + startChildrenLayout + nativeEvent.layout.height - startChildrenLayout;
        animate(toValue);
    };

    const onAccordionPress = () => {
        const newCollapsed = !collapsed;
        if (typeof onToggle === "function") {
            onToggle(line, newCollapsed);
        }
        const toValue = newCollapsed ? lineHeight : lineHeight + lineChildren.length * lineHeight;
        animate(toValue);
        setCollapsed(newCollapsed);
    };

    const LeftParentComponent = lineToggle ? TouchableOpacity : View;
    return (
        <Css classes={["opt-collapse-item-container", ...classesContainer]} cssKey={key} active={lineActive}>
            <Animated.View style={{ height: animatedHeight, overflow: "hidden" }}>
                <View
                    style={[
                        styles.line,
                        lineActive && {
                            borderTopEndRadius: CARD_RADIUS,
                            borderBottomEndRadius: CARD_RADIUS,
                            backgroundColor: LiveConfig.getSupportColor()
                        }
                    ]}>
                    <LeftParentComponent onPress={onAccordionPress} style={styles.lineLeft}>
                        {renderLineLeft && renderLineLeft(line, collapsed, lineActive)}
                    </LeftParentComponent>
                    <View style={styles.lineRight}>
                        {hasChildren ? (
                            <Css
                                classes={["opt-collapse-icon-container", ...classesIconContainer]}
                                cssKey={key}
                                active={!collapsed}
                                toggle>
                                <TouchableOpacity onPress={onAccordionPress} style={styles.accordionContainer}>
                                    <Css
                                        classes={["opt-collapse-icon", ...classesIcon]}
                                        cssKey={key}
                                        active={!collapsed}
                                        toggle>
                                        {renderLineAccordion ? (
                                            renderLineAccordion(line, collapsed)
                                        ) : (
                                            <CollapseAccordion collapsed={collapsed} />
                                        )}
                                    </Css>
                                </TouchableOpacity>
                            </Css>
                        ) : null}
                    </View>
                </View>
                {hasChildren ? (
                    <Css
                        classes={["opt-collapse-children", ...classesChildrenContainer]}
                        cssKey={key}
                        active={lineActive}>
                        <View onLayout={onChildrenLayout} style={{ marginLeft: lineLevel * 10 }}>
                            {lineChildren.map((lineChild) => {
                                if (typeof renderLineChild === "function") {
                                    return renderLineChild(lineChild, collapsed, lineLevel + 1);
                                }
                                return null;
                            })}
                        </View>
                    </Css>
                ) : null}
            </Animated.View>
        </Css>
    );
}

CollapseLine.propTypes = {
    //default collapse state
    lineCollapsed: PropTypes.bool,
    //line object, is always return like first element inside callbacks
    line: PropTypes.object,
    //apply custom style if line is active
    lineActive: PropTypes.bool,
    //apply a left margin, increasead by level depth
    lineLevel: PropTypes.number,
    //enable collapse event on line press
    lineToggle: PropTypes.bool,
    //custom line height
    lineHeight: PropTypes.number,
    //array of children objects
    lineChildren: PropTypes.array,
    //render a Component for left area
    renderLineLeft: PropTypes.func,
    //render custom accordion
    renderLineAccordion: PropTypes.func,
    //render child
    renderLineChild: PropTypes.func,
    //called on collapse state change
    onToggle: PropTypes.func,
    //customize views style
    css: PropTypes.shape({
        key: PropTypes.string,
        classesContainer: PropTypes.arrayOf(PropTypes.string),
        classesIconContainer: PropTypes.arrayOf(PropTypes.string),
        classesIcon: PropTypes.arrayOf(PropTypes.string),
        classesChildrenContainer: PropTypes.arrayOf(PropTypes.string)
    })
};

CollapseLine.defaultProps = {
    lineCollapsed: true,
    lineActive: false,
    lineToggle: false,
    lineHeight: collapseItemHeight(),
    lineLevel: 0,
    lineChildren: [],
    css: {}
};

export { collapseItemHeight };
export default CollapseLine;
