class CssStyle {
    /**
     * remove = true delete the key from style because on android old if we pass an unsupported style the app could crash
     * for example textTransform
     */
    static getAttributeFromStyle(key, style = {}, defaultValue = undefined, remove = false): String {
        if (key) {
            if (Array.isArray(style)) {
                for (let i = 0; i < style.length; i++) {
                    //!avoid to pass the default value to the recursion not to stop at the first element of the array
                    const attribute = CssStyle.getAttributeFromStyle(key, style[i], undefined, remove);
                    if (attribute !== undefined) {
                        return attribute;
                    }
                }
            }
            if (typeof style === "object") {
                if (style[key] !== undefined) {
                    const value = style[key];
                    if (remove) {
                        delete style[key];
                    }
                    return value;
                }
            }
        }
        return defaultValue;
    }

    static getFilterAttributeStyle(style = {}, defaultValue = undefined): String {
        return CssStyle.getAttributeFromStyle("filter", style, defaultValue);
    }

    static getTintColorAttributeStyle(style = {}, defaultValue = undefined): String {
        return CssStyle.getAttributeFromStyle("tintColor", style, defaultValue);
    }

    static deleteKeysFromStyle(style, keys = []) {
        if (style === null || style === undefined) {
            return style;
        }
        if (Array.isArray(style)) {
            return style.map((item) => CssStyle.deleteKeysFromStyle(item, keys));
        }
        if (typeof style === "object") {
            const newStyle = {};
            Object.keys(style).forEach((attribute) => {
                if (!keys.includes(attribute)) {
                    newStyle[attribute] = style[attribute];
                }
            });
            return newStyle;
        }
        return style;
    }

    static parseFilterValue(filter: String, filterType: String): Number {
        const regex = new RegExp(`${filterType}\\(([0-9]{1,3})%\\)`);
        if (filter) {
            const result = regex.exec(filter);
            if (result && result[1]) {
                return parseInt(result[1], 10);
            }
        }
        return undefined;
    }

    /**
     * react give high priority to attribute with left, right, top, bottom constaint
     * instead of generic, this mehtod doesn't support it for the moment
     * @param {*} style
     * @param {*} constraint
     * @param {*} dpiDivider
     */
    static getConstraintInsideStyle(style, constraint, dpiDivider) {
        const constraints = {
            start: 0,
            end: 0,
            top: 0,
            bottom: 0
        };
        Object.keys(style).forEach((key) => {
            if (key.startsWith(constraint)) {
                if (key.includes("Left") || key.includes("Start")) {
                    constraints.start = style[key];
                } else if (key.includes("Right") || key.includes("End")) {
                    constraints.end = style[key];
                } else if (key.includes("Top")) {
                    constraints.top = style[key];
                } else if (key.includes("Bottom")) {
                    constraints.bottom = style[key];
                } else if (key.includes("Horizontal")) {
                    constraints.start = style[key] / 2;
                    constraints.end = style[key] / 2;
                } else if (key.includes("Vertical")) {
                    constraints.top = style[key] / 2;
                    constraints.bottom = style[key] / 2;
                } else if (key === constraint || key === `${constraint}Width`) {
                    //for example margin or padding or borderWidth
                    constraints.start = style[key];
                    constraints.end = style[key];
                    constraints.top = style[key];
                    constraints.bottom = style[key];
                }
            }
        });
        return {
            start: constraints.start / dpiDivider,
            end: constraints.end / dpiDivider,
            top: constraints.top / dpiDivider,
            bottom: constraints.bottom / dpiDivider
        };
    }

    static updateSizeByApplingStyle(size, style) {
        if (!size || !style) {
            return size;
        }
        const margin = CssStyle.getConstraintInsideStyle(style, "margin", 1);
        const padding = CssStyle.getConstraintInsideStyle(style, "padding", 1);
        const border = CssStyle.getConstraintInsideStyle(style, "border", 1);

        const horizontalConstraint =
            margin.start + margin.end + padding.start + padding.end + border.start + border.end;
        const verticalConstraint =
            margin.top + margin.bottom + padding.top + padding.bottom + border.top + border.bottom;
        return {
            width: Math.max(0, size.width - horizontalConstraint),
            height: Math.max(0, size.height - verticalConstraint)
        };
    }

    static grayscaleValue(style, defaultValue = 100) {
        const value = CssStyle.parseFilterValue(CssStyle.getFilterAttributeStyle(style), "grayscale");
        if (value !== undefined) {
            return value;
        }
        return defaultValue;
    }

    static styleToWebInnerStyle(style = {}): String {
        return Object.keys(style).reduce((result, key) => {
            return `${result} ${key}:${style[key]};`;
        }, "");
    }
}
export default CssStyle;
