import React, { useCallback, useState } from "react";
import PropTypes from "prop-types";
import { useNavigationState } from "@react-navigation/native";
import Route from "../../models/route";
import { collapseRoute } from "./drawer-utils";
import { getActiveRouteFromState } from "../../data/navigation-config";
import CollapseLine, { collapseItemHeight } from "../CollapseLine/collapse-line";
import DrawerItemLeft from "./drawer-item-text";

/**
 * render drawer item with animation on collapse function
 * @param {*} props
 * @returns
 */
function DrawerItem(props) {
    const { onPress, route, active, navigation, collapsable }: { route: Route } = props;
    const startHeight = collapseItemHeight();

    const onItemPress = useCallback(() => {
        if (typeof onPress === "function") {
            onPress(route);
        }
    }, []);

    const onToggle = useCallback((item: Route, collapsed: Boolean) => {
        //collapse all nested children
        if (collapsed) {
            collapseRoute(route);
        } else {
            route.setCollapsed(false);
        }
    }, []);

    return (
        <CollapseLine
            lineCollapsed={route.isCollapsed()}
            line={route}
            lineActive={active}
            lineLevel={route.getLevel() + 1}
            lineHeight={startHeight}
            lineChildren={collapsable ? route.getRoutes() : []}
            renderLineLeft={(item: Route, collapsed: Boolean, isActive: Boolean) => {
                return <DrawerItemLeft route={item} onPress={onItemPress} active={isActive} />;
            }}
            renderLineChild={(item: Route, collapsed: Boolean) => {
                //the key with collapsed is usefull to redraw content when parent state change
                return (
                    <DrawerItemWithState
                        key={`drawer_item_${item.getRoute()}_collapsed_${collapsed}`}
                        onPress={onPress}
                        route={item}
                        navigation={navigation}
                    />
                );
            }}
            onToggle={onToggle}
            css={{
                key: route.getRoute(),
                classesContainer: ["opt-drawer-item-container"],
                classesIconContainer: ["opt-drawer-item-icon-container"],
                classesIcon: ["opt-drawer-item-icon"],
                classesChildrenContainer: ["opt-drawer-item-children"]
            }}
        />
    );
}

DrawerItem.propTypes = {
    onPress: PropTypes.func.isRequired,
    route: PropTypes.instanceOf(Route).isRequired,
    active: PropTypes.bool,
    collapsable: PropTypes.bool
};

DrawerItem.defaultProps = {
    collapsable: true
};

/**
 * add active prop to drawer item based on route state
 * @param {*} props
 * @returns
 */
function DrawerItemWithState(props) {
    const { route, navigation } = props;
    const [active, setActive] = useState(getActiveRouteFromState(navigation.getState()) === route.getRoute());

    useNavigationState((state) => {
        if (state) {
            const activeRoute = getActiveRouteFromState(state);
            const newActive = activeRoute === route.getRoute();
            if (newActive !== active) {
                setActive(newActive);
            }
        }
    });

    return <DrawerItem {...props} active={active} />;
}

DrawerItemWithState.propTypes = {
    ...DrawerItem.propTypes
};

export default DrawerItemWithState;
