import React, { useState } from "react";
import PropTypes from "prop-types";
import { Modal, StyleSheet, View } from "react-native";
import { colors } from "../../utils/colors";
import { commonStyles, DRAWER_PERMANENT } from "../Css/common-styles";
import LiveConfig from "../../data/live-config";
import Edition from "../../models/edition";
import EditionsSecondaryModalHeader from "./editions-secondary-modal-header";
import { SelectedEditionsContext } from "../../data/user-config";
import { Css } from "../Css/css";
import { SearchInput } from "../Search/search-input";
import EditionsHelper from "../../helpers/editions-helper";
import EditionList from "../List/edition-list";
import EditionsTags from "./editions-tags";

const MARGIN = 10;

const styles = StyleSheet.create({
    modal: {
        margin: 0
    },
    container: {
        flex: 1,
        backgroundColor: colors.transparent600,
        alignItems: "center",
        padding: 50
    },
    content: {
        padding: 0,
        backgroundColor: "white",
        flex: 1,
        margin: MARGIN
    },
    touchable: {
        height: 100,
        width: 100,
        justifyContent: "center",
        alignItems: "center"
    },
    search: {
        backgroundColor: colors.white,
        borderWidth: 0,
        borderBottomWidth: 1
    }
});

function filterEditions(editions = [], filter = "") {
    if (filter && filter.length > 2) {
        const result = [];
        for (let i = 0; i < editions.length; i++) {
            const splitted = editions[i].getName().split("_");
            let lastName;
            if (splitted) {
                lastName = splitted[splitted.length - 1];
            }
            if (!lastName) {
                lastName = editions[i].getName();
            }
            if (lastName.trim().toLowerCase().search(filter.toLowerCase()) >= 0) {
                result.push(editions[i]);
            }
            const group = editions[i].getGroup();
            if (group && group.length > 0) {
                const filteredEditions = filterEditions(group, filter);
                if (filteredEditions && filteredEditions.length > 0) {
                    result.push(...filteredEditions);
                }
            }
        }
        return result;
    }
    if (editions.length === 1 && editions[0].getGroup().length) {
        return editions[0].getGroup();
    }
    return editions;
}

/**
 * modal that display an editions list to select preferred editions
 * @param {*} props
 * @returns
 */
function EditionsSecondaryModal(props) {
    const { onClose, visible, editions } = props;
    const [searchText, setSearchText] = useState("");

    const onHide = () => {
        setSearchText("");
        if (typeof onClose === "function") {
            onClose();
        }
    };

    /**
     * we must merge all editions inside a unique to have just one FlatList
     */
    const filteredEditions = filterEditions(editions, searchText);
    const mergeEdition = new Edition();
    mergeEdition.setName("merged");
    mergeEdition.setGroup(filteredEditions);

    return (
        <Css classes={["opt-editions-secondary-modal"]}>
            <Modal
                animationType="fade"
                onRequestClose={onHide}
                onDismiss={onHide}
                visible={visible}
                transparent
                style={styles.modal}>
                <Css classes={["opt-editions-secondary-container"]}>
                    <View style={styles.container}>
                        <Css classes={["opt-editions-secondary-content"]}>
                            <View
                                style={[
                                    commonStyles.modalContent,
                                    styles.content,
                                    {
                                        width: Math.min(DRAWER_PERMANENT, LiveConfig.getWindow().width) - MARGIN * 2
                                    }
                                ]}>
                                <EditionsSecondaryModalHeader
                                    title={LiveConfig.getLocale().strings("str_modal_header_preferred_edition")}
                                    onClose={onHide}
                                    useDoneButton={false}
                                />
                                <SearchInput
                                    style={[commonStyles.shadow, styles.search]}
                                    placeholder={LiveConfig.getLocale().strings("str_search")}
                                    onChangeText={setSearchText}
                                />

                                <View style={[commonStyles.container, commonStyles.background]}>
                                    <SelectedEditionsContext.Consumer>
                                        {({ editions: selectedEditions, setEdition }) => (
                                            <>
                                                <EditionsTags
                                                    tags={selectedEditions
                                                        .map((editionName) => {
                                                            return EditionsHelper.getEditionByName(
                                                                editions,
                                                                editionName
                                                            );
                                                        })
                                                        .filter((edition) => edition !== undefined)}
                                                    onTagPress={setEdition}
                                                />
                                                <EditionList
                                                    key={`${searchText}`}
                                                    horizontal={false}
                                                    edition={mergeEdition}
                                                    onFavoritePress={(item: Edition) => {
                                                        setEdition(item);
                                                        //! if we close the modal we could remove Context Consumer
                                                        onHide();
                                                    }}
                                                    isSelected={(item: Edition) => {
                                                        return selectedEditions.includes(item.getName());
                                                    }}
                                                    onlyGroup
                                                />
                                            </>
                                        )}
                                    </SelectedEditionsContext.Consumer>
                                </View>
                            </View>
                        </Css>
                    </View>
                </Css>
            </Modal>
        </Css>
    );
}

EditionsSecondaryModal.propTypes = {
    onClose: PropTypes.func.isRequired,
    visible: PropTypes.bool,
    editions: PropTypes.arrayOf(PropTypes.instanceOf(Edition))
};

EditionsSecondaryModal.defaultProps = {
    visible: false
};

export default EditionsSecondaryModal;
