import React, { useCallback, useState } from "react";
import PropTypes from "prop-types";
import { Touchable, TouchableOpacity, View } from "react-native";
import IssueList from "../List/issue-list";
import Edition from "../../models/edition";
import EditionsHelper from "../../helpers/editions-helper";
import EditionsSecondaryEmpty from "./editions-secondary-empty";
import EditionsSecondaryModal from "./editions-secondary-modal";
import LiveConfig from "../../data/live-config";
import Issue from "../../models/issue";
import { MAX_SCREEN_WIDTH, NEW_LINE } from "../Css/common-styles";
import IssuesGrid from "../Grid/issues-grid";
import EditionDayHeader from "../EditionDayPage/edition-day-header";
import IcPlus from "../Icon/ic-plus";
import { colors } from "../../utils/colors";

/**
 * display secondary editions add button
 * @param {*} props
 * @returns
 */
function EditionsSecondary(props) {
    const { editions, skipIssues, mainEditionName, closestIssue, issueSize, issueCallbacks, headerColor } = props;
    const [showModal, setShowModal] = useState(false);

    if (!editions || editions.length === 0) {
        return null;
    }

    const { selectedEditions } = LiveConfig.getSettings();
    const { orderEditions } = LiveConfig.getPSetup();

    const show = useCallback(() => {
        setShowModal(true);
    });

    const hide = useCallback(() => {
        setShowModal(false);
    });

    const data = [];

    if (selectedEditions) {
        //if vertical prepare a unique vertical list with all the secondary editions
        const secondaryEditions = selectedEditions.reduce((current, editionName: String) => {
            if (editionName === mainEditionName) {
                return null;
            }
            const edition: Edition = EditionsHelper.getEditionByName(editions, editionName);
            if (edition) {
                current.push(edition);
            }
            return current;
        }, []);

        data.push(
            ...EditionsHelper.getIssuesFromPublications(secondaryEditions, mainEditionName, {
                forceShortcut: true,
                closestIssue
            })
                .map((issue: Issue) => {
                    const edition = EditionsHelper.getEditionByName(secondaryEditions, issue.getEdition());
                    return {
                        issue,
                        label: `${edition ? edition.getLabel() : ""}${NEW_LINE}-${NEW_LINE}issue`
                    };
                })
                .filter(({ issue }) => !skipIssues.length || skipIssues.find((toSkip: Issue) => !issue.equals(toSkip)))
                .sort((itemA, itemB) => {
                    const indexA = orderEditions.indexOf(EditionsHelper.getMainEdition(itemA.issue.getEdition()));
                    const indexB = orderEditions.indexOf(EditionsHelper.getMainEdition(itemB.issue.getEdition()));
                    return indexA - indexB;
                })
        );
    }

    return (
        <>
            <EditionDayHeader.Container>
                <EditionDayHeader.Body
                    style={{ justifyContent: "space-between" }}
                    text={LiveConfig.getLocale().strings("str_modal_header_preferred_edition")}
                    textColor={headerColor}>
                    {/*<TouchableOpacity onPress={show}>
                            <IcPlus fill={headerColor} />
                    </TouchableOpacity>*/}
                </EditionDayHeader.Body>
            </EditionDayHeader.Container>
            <View style={{ width: "100%", maxWidth: MAX_SCREEN_WIDTH }}>
                <IssuesGrid
                    data={data}
                    issueCallbacks={issueCallbacks}
                    issueSize={issueSize}
                    EmptyComponent={<EditionsSecondaryEmpty onPress={show} color={headerColor} />}
                />
            </View>
            <EditionsSecondaryModal editions={editions} onClose={hide} visible={showModal} />
        </>
    );
}

EditionsSecondary.defaultProps = {
    issueSize: {
        width: 150,
        height: 200
    },
    closestIssue: "",
    skipIssues: [],
    headerColor: colors.black
};

EditionsSecondary.propTypes = {
    headerColor: PropTypes.string,
    mainEditionName: PropTypes.string,
    closestIssue: PropTypes.string,
    editions: PropTypes.arrayOf(PropTypes.instanceOf(Edition)),
    skipIssues: PropTypes.arrayOf(PropTypes.instanceOf(Issue)),
    issueSize: PropTypes.object,
    ...IssueList.propTypes
};

export default EditionsSecondary;
