import React, { useRef, useState } from "react";
import PropTypes from "prop-types";
import { StyleSheet, View, Platform } from "react-native";
import { Css } from "../Css/css";
import TaText from "../Tecnavia/ta-text";
import { colors } from "../../utils/colors";
import { CARD_MARGIN, CARD_PADDING, CARD_RADIUS, NEW_LINE } from "../Css/common-styles";
import DownloadedCheck from "./downloaded-check";

export const BADGE_SIZE = 40;
const BADGE_BOTTOM_CONSTRAINT = BADGE_SIZE / 2 + CARD_MARGIN;

const styles = StyleSheet.create({
    container: {
        position: "absolute",
        flexDirection: "row",
        alignItems: "center",
        start: CARD_PADDING,
        end: CARD_PADDING,
        padding: CARD_PADDING,
        bottom: -(BADGE_SIZE / 2),
        borderWidth: 0.5,
        borderColor: colors.grey300,
        borderRadius: CARD_RADIUS,
        backgroundColor: colors.white,
        minHeight: BADGE_SIZE
    },
    footer: {
        flex: 1,
        alignItems: "center",
        flexDirection: "column",
        justifyContent: "space-between"
    },
    footerText: {
        color: colors.black,
        fontWeight: "600",
        textTransform: "uppercase",
        ...Platform.select({
            web: {
                whiteSpace: "normal",
                overflowWrap: "normal"
            }
        })
    },
    footerCenter: {
        textAlign: "center"
    },
    footerLeft: {
        textAlign: "left"
    }
});

function Badge(props) {
    const { minFontSize, cssPrefix, text = "", showDownloadCheck, adaptFontSize, adaptWidth, style } = props;
    const [evaluated, setEvaluated] = useState(false);
    const [fontSize, setFontSize] = useState(
        Platform.select({
            native: 12,
            default: 13
        })
    );
    const [paddingHorizontal, setPaddingHorizontal] = useState(CARD_PADDING * 2);
    const [paddingInternal, setPaddingInternal] = useState(CARD_PADDING);
    const [enableSplitOnFirstWhiteSpace, setEnableSplitOnFirstWhiteSpace] = useState(false);

    const textLayout = useRef(undefined);
    const parentLayout = useRef(undefined);

    const evaluateFontSize = () => {
        if (textLayout.current && parentLayout.current && !evaluated) {
            const paddingAttemptCompleted = adaptWidth && paddingHorizontal === 0 && paddingInternal < 5;

            if (textLayout.current.width <= parentLayout.current.width - paddingInternal * 2) {
                setEvaluated(true);
            } else if (adaptFontSize || paddingAttemptCompleted) {
                if (text.trim().indexOf(" ") >= 0 && !enableSplitOnFirstWhiteSpace) {
                    setEnableSplitOnFirstWhiteSpace(true);
                } else {
                    const newFontSize = fontSize - 1;
                    if (newFontSize === minFontSize) {
                        setEvaluated(true);
                    }
                    setFontSize(newFontSize);
                }
            } else if (adaptWidth && paddingHorizontal === 0) {
                setPaddingInternal(paddingInternal - 1);
            } else if (adaptWidth) {
                setPaddingHorizontal(paddingHorizontal < 1 ? 0 : paddingHorizontal / 2);
            }
        }
    };

    let textLines = text.trim().split(NEW_LINE);
    if (enableSplitOnFirstWhiteSpace) {
        textLines = textLines.reduce((current, line) => {
            const lastIndex = line.indexOf(" ");
            const before = line.slice(0, lastIndex);
            const after = line.slice(lastIndex + 1);

            current.push(before, after);

            return current;
        }, []);
    }

    return (
        <Css classes={["opt-issue-item-footer-container"]} cssKey={cssPrefix}>
            <View
                style={[
                    styles.container,
                    style,
                    { start: paddingHorizontal / 2, end: paddingHorizontal / 2, padding: paddingInternal }
                ]}>
                <View
                    style={styles.footer}
                    onLayout={({ nativeEvent }) => {
                        parentLayout.current = nativeEvent.layout;
                        evaluateFontSize();
                    }}>
                    <Css classes={["opt-issue-item-footer-text"]} cssKey={cssPrefix}>
                        <View>
                            {textLines.map((item, index) => {
                                return (
                                    <TaText
                                        key={`footer_text_${index}_${index}`}
                                        numberOfLines={3}
                                        style={[styles.footerText, styles.footerCenter, { fontSize }]}
                                        onLayout={({ nativeEvent }) => {
                                            textLayout.current = nativeEvent.layout;
                                            evaluateFontSize();
                                        }}>
                                        {item}
                                    </TaText>
                                );
                            })}
                        </View>
                    </Css>
                </View>
                {showDownloadCheck ? <DownloadedCheck size={16} cssKey="badge" /> : null}
            </View>
        </Css>
    );
}

Badge.defaultProps = {
    minFontSize: 9,
    showDownloadCheck: false,
    adaptFontSize: false,
    adaptWidth: true
};

Badge.propTypes = {
    showDownloadCheck: PropTypes.bool,
    minFontSize: PropTypes.number,
    cssPrefix: PropTypes.string,
    text: PropTypes.string,
    adaptFontSize: PropTypes.bool,
    adaptWidth: PropTypes.bool,
    style: PropTypes.oneOfType([PropTypes.number, PropTypes.object, PropTypes.array])
};

export { BADGE_BOTTOM_CONSTRAINT };
export default Badge;
