import React, { useCallback, useState } from "react";
import { StyleSheet, View } from "react-native";
import PropTypes from "prop-types";
import HorizontalList from "./horizontal-list";
import VerticalList from "./vertical-list";
import { THUMBNAIL_HEIGHT, updateSize } from "./list-utils";
import { commonStyles } from "../Css/common-styles";
import CssStyle from "../Css/css-styles";
import { CssHelper } from "../Css/css";

const styles = StyleSheet.create({
    container: {
        flex: 1
    }
});

/**
 * component that choose wich kind of list render
 * evaluate also item size by checking first issue aspect ratio and parent constraint
 * @param {*} props
 * @returns
 */
function LayoutList(props) {
    const {
        horizontal,
        aspectRatio,
        itemHeight,
        containerStyle,
        renderItem,
        renderVerticalItem,
        renderHorizontalItem,
        cssListClasses,
        itemStyle,
        ...rest
    } = props;
    const [layout, setLayout] = useState();

    const onLayout = useCallback(({ nativeEvent }) => {
        setLayout(nativeEvent.layout);
    });

    if (!aspectRatio) {
        return null;
    }

    let ToRender = null;
    if (layout) {
        const cssListContainer = CssHelper.flatten(
            [styles.container, commonStyles.margin],
            ["opt-list-container", ...cssListClasses]
        );
        //update constraints to set correct item size on window resize
        const constraints = CssStyle.updateSizeByApplingStyle(layout, cssListContainer);

        const itemFullStyle = CssHelper.flatten([{ height: itemHeight }, itemStyle]);
        const itemSize = updateSize({ height: itemFullStyle.height, aspectRatio, constraints });

        const ListComponent = horizontal ? HorizontalList : VerticalList;
        const listConstraintRender = horizontal ? renderHorizontalItem : renderVerticalItem;

        ToRender = (
            <ListComponent
                itemSize={itemSize}
                {...rest}
                renderItem={({ item, index, style, size }) => {
                    const itemProps = { item, index, style, size };
                    let Rendered = null;
                    if (typeof listConstraintRender === "function") {
                        Rendered = listConstraintRender(itemProps);
                    }
                    if (typeof renderItem === "function") {
                        Rendered = renderItem(itemProps);
                    }
                    if (Rendered) {
                        return <View style={style}>{Rendered}</View>;
                    }
                    return null;
                }}
            />
        );
    }

    return (
        <View style={containerStyle || styles.container} onLayout={onLayout}>
            {ToRender}
        </View>
    );
}

LayoutList.propTypes = {
    renderItem: PropTypes.func,
    renderVerticalItem: PropTypes.func,
    renderHorizontalItem: PropTypes.func,
    aspectRatio: PropTypes.number,
    itemHeight: PropTypes.number,
    keyExtractor: PropTypes.func.isRequired,
    data: PropTypes.array.isRequired,
    cssListClasses: PropTypes.array,
    horizontal: PropTypes.bool,
    removeClippedSubviews: PropTypes.bool,
    initialNumToRender: PropTypes.number,
    containerStyle: PropTypes.oneOfType([PropTypes.array, PropTypes.object, PropTypes.number]),
    itemStyle: PropTypes.oneOfType([PropTypes.array, PropTypes.object, PropTypes.number])
};

LayoutList.defaultProps = {
    horizontal: true,
    initialNumToRender: 5,
    removeClippedSubviews: true,
    cssListClasses: [],
    itemHeight: THUMBNAIL_HEIGHT
};

export default LayoutList;
