import React, { Component } from "react";
import PropTypes from "prop-types";
import { Keyboard, View, TextInput, StyleSheet, TouchableOpacity } from "react-native";
import { colors } from "../../utils/colors";
import { Css, CssHelper } from "../Css/css";
import CssStyle from "../Css/css-styles";
import IcCloseCircle from "../Icon/ic-close-circle";

const styles = StyleSheet.create({
    container: {
        height: 40,
        flexDirection: "row",
        justifyContent: "flex-start",
        alignItems: "center"
    },
    input: {
        flex: 1,
        paddingTop: 5,
        paddingBottom: 5,
        paddingRight: 10,
        paddingLeft: 10,
        fontSize: 13
    },
    searchClose: {
        margin: 5,
        justifyContent: "center"
    }
});

export class SearchInput extends Component {
    static propTypes = {
        afterFocus: PropTypes.func,
        afterSearch: PropTypes.func,
        autoCapitalize: PropTypes.string,
        autoFocus: PropTypes.bool,
        beforeFocus: PropTypes.func,
        beforeSearch: PropTypes.func,
        blurOnSubmit: PropTypes.bool,
        editable: PropTypes.bool,
        keyboardShouldPersist: PropTypes.bool,
        keyboardType: PropTypes.string,
        onBlur: PropTypes.func,
        onChangeText: PropTypes.func,
        onFocus: PropTypes.func,
        onSearch: PropTypes.func,
        placeholder: PropTypes.string,
        textColor: PropTypes.string,
        style: PropTypes.any
    };

    static defaultProps = {
        editable: true,
        blurOnSubmit: false,
        keyboardShouldPersist: false,
        textColor: colors.black,
        autoFocus: false
    };

    constructor(props) {
        super(props);

        const { placeholder } = this.props;
        /**
         * local variables
         */
        this.placeholder = placeholder || "Search";

        /**
         * functions
         */
        this.onRef = this._onRef.bind(this);
        this.onFocus = this._onFocus.bind(this);
        this.onSearch = this._onSearch.bind(this);
        this.onChangeText = this._onChangeText.bind(this);
        this.onClearSearch = this._onClearSearch.bind(this);

        this.state = {
            keyword: ""
        };
    }

    /**
     * onSearch
     * async await
     */
    async _onSearch() {
        const { beforeSearch, keyboardShouldPersist, onSearch, afterSearch } = this.props;
        const { keyword } = this.state;
        if (beforeSearch) {
            await beforeSearch(keyword);
        }
        if (keyboardShouldPersist === false) {
            await Keyboard.dismiss();
        }
        if (onSearch) {
            await onSearch(keyword);
        }
        if (afterSearch) {
            await afterSearch(keyword);
        }
    }

    /**
     * onChangeText
     * async await
     */
    async _onChangeText(text) {
        const { onChangeText } = this.props;

        if (onChangeText) {
            await onChangeText(text);
        }

        await this.setState({ keyword: text });
    }

    _onClearSearch() {
        this._onChangeText("");
    }

    _onRef(inputKeyword) {
        this.inputKeyword = inputKeyword;
    }

    /**
     * onFocus
     * async await
     */
    async _onFocus() {
        const { beforeFocus, onFocus, afterFocus } = this.props;
        const { keyword } = this.state;
        if (beforeFocus) {
            await beforeFocus();
        }

        if (this.inputKeyword && !this.inputKeyword.isFocused()) {
            await this.inputKeyword.focus();
        }
        if (onFocus) {
            await onFocus(keyword);
        }
        if (afterFocus) {
            await afterFocus();
        }
    }

    render() {
        const { editable, onBlur, blurOnSubmit, keyboardType, autoCapitalize, autoFocus, textColor, style } =
            this.props;
        const { keyword } = this.state;

        const searchTextStyle = CssHelper.getStyleObjByCssClasses(["opt-search-input"]);
        const color = CssStyle.getAttributeFromStyle("color", searchTextStyle, textColor);

        return (
            <Css classes={["opt-search-container"]}>
                <View style={[styles.container, style]}>
                    <TextInput
                        ref={this.onRef}
                        style={[styles.input, searchTextStyle]}
                        editable={editable}
                        value={keyword}
                        onChangeText={this.onChangeText}
                        placeholder={this.placeholder}
                        placeholderTextColor={color}
                        onSubmitEditing={this.onSearch}
                        autoCorrect={false}
                        blurOnSubmit={blurOnSubmit}
                        returnKeyType="search"
                        keyboardType={keyboardType || "default"}
                        autoCapitalize={autoCapitalize}
                        onBlur={onBlur}
                        onFocus={this.onFocus}
                        underlineColorAndroid="transparent"
                        autoFocus={autoFocus}
                    />
                    {keyword ? (
                        <TouchableOpacity style={styles.searchClose} onPress={this.onClearSearch}>
                            <IcCloseCircle />
                        </TouchableOpacity>
                    ) : null}
                </View>
            </Css>
        );
    }
}
