import React, { useEffect, useRef, useState } from "react";
import { Animated, Dimensions, View, TouchableOpacity, StyleSheet, I18nManager } from "react-native";
import PropTypes from "prop-types";
import { commonStyles } from "../Css/common-styles";
import { Css } from "../Css/css";

const SCREEN_WIDTH = Dimensions.get("window").width;
const DISTANCE_BETWEEN_TABS = 40;
const MIN_TAB_WIDTH = 100;

const styles = StyleSheet.create({
    container: {
        backgroundColor: "black",
        height: 50
    },
    scrollview: {
        flexDirection: "row",
        flex: 1
    },
    button: {
        alignItems: "center",
        justifyContent: "center"
    },
    buttonContainer: {
        minWidth: MIN_TAB_WIDTH,
        paddingHorizontal: DISTANCE_BETWEEN_TABS / 2
    },
    indicator: {
        backgroundColor: "white",
        bottom: 0,
        height: 2,
        left: 0,
        position: "absolute",
        right: 0,
        // this must be 1 for the scaleX animation to work properly
        width: 1
    },
    text: {
        color: "white",
        fontSize: 14,
        textAlign: "center"
    }
});

function TabBar(props) {
    const { state, descriptors, navigation, position, textColor, backgroundColor, screenWidth } = props;
    const [widths, setWidths] = useState([]);
    const scrollViewRef = useRef(null);
    const tempWidths = useRef([]);
    const transform = [];
    const inputRange = state.routes.map((route, index) => index);

    // keep a ref to easily scroll the tab bar to the focused label
    const outputRangeRef = useRef([]);

    const getTranslateX = (xPosition, xRoutes, xWidths) => {
        const outputRange = xRoutes.reduce((result, item, index) => {
            if (index === 0) {
                return [xWidths[0] / 2];
            }
            return [...result, result[index - 1] + xWidths[index - 1] / 2 + xWidths[index] / 2];
        }, []);

        outputRangeRef.current = outputRange;
        const translateX = xPosition.interpolate({
            inputRange,
            outputRange,
            extrapolate: "clamp"
        });
        return Animated.multiply(translateX, I18nManager.isRTL ? -1 : 1);
    };

    // compute translateX and scaleX because we cannot animate width directly
    if (state.routes.length > 1 && widths.length === state.routes.length && !widths.includes(undefined)) {
        const translateX = getTranslateX(position, state.routes, widths);
        transform.push({
            translateX
        });
        const outputRange = inputRange.map((_, i) => widths[i]);
        transform.push({
            scaleX:
                state.routes.length > 1
                    ? position.interpolate({
                          inputRange,
                          outputRange,
                          extrapolate: "clamp"
                      })
                    : outputRange[0]
        });
    }

    // scrolls to the active tab label when a new tab is focused
    useEffect(() => {
        if (state.routes.length > 1 && widths.length === state.routes.length && !widths.includes(undefined)) {
            if (state.index === 0) {
                scrollViewRef.current?.scrollTo({
                    x: 0
                });
            } else {
                // keep the focused label at the center of the screen
                scrollViewRef.current?.scrollTo({
                    x: outputRangeRef.current[state.index] - screenWidth / 2
                });
            }
        }
    }, [state.index, state.routes.length, widths]);

    // get the label widths on mount
    const onLayout = (event, index) => {
        const { width } = event.nativeEvent.layout;
        tempWidths.current[index] = width;
        setWidths([...tempWidths.current]);
    };

    // basic labels as suggested by react navigation
    const labels = state.routes.map((route, index) => {
        const { options } = descriptors[route.key];
        const { tabBarLabel } = options;
        const label = tabBarLabel;
        const isFocused = state.index === index;

        const onPress = () => {
            const event = navigation.emit({
                type: "tabPress",
                target: route.key,
                canPreventDefault: true
            });

            if (!isFocused && !event.defaultPrevented) {
                // The `merge: true` option makes sure that the params inside the tab screen are preserved
                // eslint-disable-next-line
                // @ts-ignore
                navigation.navigate({ name: route.name, merge: true });
            }
        };

        return (
            <Css classes={["opt-tab-item"]} key={route.key}>
                <TouchableOpacity
                    accessibilityRole="button"
                    accessibilityState={isFocused ? { selected: true } : {}}
                    accessibilityLabel={options.tabBarAccessibilityLabel}
                    onPress={onPress}
                    style={styles.button}>
                    <View onLayout={(event) => onLayout(event, index)} style={styles.buttonContainer}>
                        <Animated.Text style={[styles.text, { color: textColor }]}>{label}</Animated.Text>
                    </View>
                </TouchableOpacity>
            </Css>
        );
    });

    return (
        <View style={[styles.container, commonStyles.darkShadow, { backgroundColor }]}>
            <Css classes={["opt-tab"]}>
                <Animated.ScrollView
                    horizontal
                    ref={scrollViewRef}
                    showsHorizontalScrollIndicator={false}
                    style={[styles.scrollview]}>
                    {labels}
                    <Css classes={["opt-tab-indicator"]}>
                        <Animated.View style={[styles.indicator, { transform, backgroundColor: textColor }]} />
                    </Css>
                </Animated.ScrollView>
            </Css>
        </View>
    );
}

TabBar.propTypes = {
    state: PropTypes.object,
    descriptors: PropTypes.object,
    navigation: PropTypes.object,
    position: PropTypes.object,
    textColor: PropTypes.string,
    backgroundColor: PropTypes.string,
    screenWidth: PropTypes.number
};

TabBar.defaultProps = {
    screenWidth: SCREEN_WIDTH
};
export default TabBar;
