/* eslint-disable react/no-danger */
import React, { useEffect, useState } from "react";
import PropTypes from "prop-types";
import { WebViewProps } from "react-native-webview";
import { toMD5 } from "../../utils/strings";
import Logger from "../../utils/log";
import { EventRegister } from "../../events/manager";
import Events from "../../events/events";
import Message from "../../models/message";
import { ProgressBar } from "../Progress/progress-circle";
import { CssHelper } from "../Css/css";

/**
 * this filter works with iframe origin
 * so will be called onMessage prop on all iframe with same origin
 * otherwise you could put inside data onMessageKey attribute and just the iframe with this key become a valid listener
 * !check how find iframe that trigger event
 */
function TaWebView(props) {
    const { source, onLocal, onMessage, onMessageKey, style, active, ...rest } = props;

    const useIframe = !!source.uri;
    const iframeIdentifier = useIframe ? toMD5(source.uri) : "";
    const [loading, setLoading] = useState(useIframe);

    const onMessageListener = (e) => {
        if (active) {
            Logger.debug(`window on message from ${source.uri}`);
            EventRegister.emit(Events.WEBVIEW_ON_MESSAGE, { message: new Message(e) });
        }
    };

    const onLoad = (e) => {
        setLoading(false);
    };

    useEffect(() => {
        const iframeJsInjectId = EventRegister.on(Events.IFRAME_JS_INJECTION, ({ js }) => {
            if (iframeIdentifier) {
                document.getElementById(iframeIdentifier).contentWindow.eval(js);
            }
        });
        window.addEventListener("message", onMessageListener);
        return () => {
            EventRegister.rm(iframeJsInjectId);
            window.removeEventListener("message", onMessageListener);
        };
    }, []);

    if (useIframe) {
        return (
            <div style={CssHelper.flatten([{ width: "100%" }, style])} {...rest}>
                {loading ? (
                    <ProgressBar
                        style={{ position: "absolute", top: 0, start: 0, end: 0 }}
                        indeterminateAnimationDuration={5000}
                    />
                ) : null}
                <iframe
                    id={iframeIdentifier}
                    title={iframeIdentifier}
                    src={source.uri}
                    frameBorder="0"
                    onLoad={onLoad}
                    style={{ width: "100%", height: "100%" }}
                />
            </div>
        );
    }

    if (source.html) {
        return (
            <div
                dangerouslySetInnerHTML={{ __html: source.html }}
                style={CssHelper.flatten([{ width: "100%" }, style])}
                {...rest}
            />
        );
    }

    return null;
}

TaWebView.propTypes = {
    active: PropTypes.bool,
    onMessage: PropTypes.func,
    //to restrict onMessage listener on a specific iframe
    onMessageKey: PropTypes.string,
    ...WebViewProps
};

TaWebView.defaultProps = {
    active: false
};

export default TaWebView;
