import { Dimensions } from "react-native";
import { HEADER_HEIGHT } from "../components/Css/common-styles";
import ColorUtils, { colors } from "../utils/colors";
import { segmentURL } from "../utils/strings";
import { init } from "./live-config-plaftorm";
import LocaleConfig from "./locale-config";
import PaperSetupConfig from "./paper-setup-config";
import UserConfig from "./user-config";
import { CssHelper } from "../components/Css/css";
import JSUtils from "../utils/js-utils";

export default class LiveConfig {
    static WebViewComponent;

    static #_initialProps: {
        supportColor: String,
        foregroundColor: String,
        apiParams: Object,
        urlParams: Object,
        cssActiveTheme: String,
        isDark: Boolean,
        isTablet: Boolean,
        mPSetup: PaperSetupConfig,
        mLocale: LocaleConfig,
        mUserSettings: UserConfig,
        server: String
    } = {};

    static #_windowSize: { width: Number, height: Number } = {};

    static #_get(prop) {
        return LiveConfig.#_initialProps ? LiveConfig.#_initialProps[prop] : undefined;
    }

    static #_set(prop, value) {
        if (LiveConfig.#_initialProps) {
            LiveConfig.#_initialProps[prop] = value;
        }
    }

    static setInitialProps(props = {}) {
        LiveConfig.#_initialProps = props;
        LiveConfig.#_windowSize = Dimensions.get("window");
        Dimensions.addEventListener("change", ({ window: wSize }) => {
            LiveConfig.#_windowSize = wSize;
            //update context style
            CssHelper.setWindowSize(LiveConfig.#_windowSize);
        });
    }

    static addProps(props = {}) {
        LiveConfig.#_initialProps = {
            ...LiveConfig.#_initialProps,
            ...props
        };

        //!check if we have a style already set by the enclosing app
        //!if so do not override the RN style object
        const currentStyleState = CssHelper.get();
        if (JSUtils.isEmpty(currentStyleState) || JSUtils.isEmpty(currentStyleState.rnStyles)) {
            CssHelper.set({
                rnStyles: !LiveConfig.getPSetup().disableRnStyles ? LiveConfig.getPSetup().rnStyles : {},
                activeTheme: LiveConfig.getActiveTheme(),
                window: LiveConfig.getWindow(),
                isTablet: LiveConfig.isTablet(),
                isDark: LiveConfig.isDark(),
                isPortrait: LiveConfig.isPortrait()
            });
        } else {
            //!app does not write the window size
            CssHelper.setWindowSize(LiveConfig.getWindow());
        }

        init({
            supportColor: LiveConfig.getSupportColor(),
            textColor: LiveConfig.getForegroundColor()
        });
    }

    static getPSetupName() {
        return LiveConfig.#_get("pSetup") ?? "";
    }

    static getMainPSetupName() {
        const pSetupName = LiveConfig.getPSetupName();
        if (pSetupName.indexOf("-xp-") !== -1) {
            const pSetupArray = pSetupName.split("-xp-");
            if (pSetupArray.length > 1) {
                return pSetupArray[0];
            }
        }

        return pSetupName;
    }

    static getServer() {
        return LiveConfig.#_get("server") ?? "";
    }

    static apiParams() {
        return LiveConfig.#_get("apiParams") ?? "";
    }

    static urlParams() {
        return LiveConfig.#_get("urlParams") ?? "";
    }

    static setUrlParams(urlParams) {
        LiveConfig.#_set("urlParams", urlParams);
    }

    static getEntryPoint() {
        return LiveConfig.#_get("entryPoint");
    }

    static getProtocol() {
        return segmentURL(LiveConfig.getServer()).getProtocol();
    }

    static isTablet() {
        return LiveConfig.#_get("isTablet") ?? false;
    }

    static isDark() {
        return LiveConfig.#_get("isDark") ?? false;
    }

    static getActiveTheme() {
        return LiveConfig.#_get("cssActiveTheme") ?? "";
    }

    static setActiveTheme(theme) {
        LiveConfig.#_set("cssActiveTheme", theme);
    }

    static isPortrait() {
        return LiveConfig.#_windowSize.width < LiveConfig.#_windowSize.height;
    }

    static getWindow() {
        return LiveConfig.#_windowSize || Dimensions.get("window");
    }

    static isWideScreen() {
        return LiveConfig.getWindow().width > LiveConfig.getWindow().height * 1.3;
    }

    static getAvailableHeight(defaultValue) {
        let value;
        try {
            value = LiveConfig.getWindow().height - HEADER_HEIGHT ?? defaultValue;
        } catch (e) {
            value = defaultValue;
        }
        return value;
    }

    static getPSetup(): PaperSetupConfig {
        return LiveConfig.#_get("mPSetup") || new PaperSetupConfig();
    }

    static getLocale(): LocaleConfig {
        return LiveConfig.#_get("mLocale") || new LocaleConfig();
    }

    static getSettings(): UserConfig {
        return LiveConfig.#_get("mUserSettings") || new UserConfig();
    }

    static getSupportColor(): String {
        const style = CssHelper.getStyleObjByCssClasses(["opt-main-header"]);
        return style.backgroundColor || LiveConfig.#_get("supportColor") || colors.black;
    }

    static getForegroundColor(): String {
        const style = CssHelper.getStyleObjByCssClasses(["opt-main-header"]);
        return (
            style.color ||
            LiveConfig.#_get("foregroundColor") ||
            ColorUtils.determineTextColor(LiveConfig.getSupportColor())
        );
    }

    static setThumbnailSize(size) {}

    static getCustomUserAgent(): String {
        return LiveConfig.#_get("customUserAgent");
    }
}
