import { getLanguagePack } from "../utils/api";
import DateUtils from "../utils/date";
import Network from "../utils/network";
import { i18nInit, i18nLocale, i18nTxt, SUPPORTED_LANGUAGES } from "../i18n/i18n-supported";
import LocaleHelper from "../helpers/locale-helper";
import Logger from "../utils/log";
import PaperSetupConfig from "./paper-setup-config";

class LocaleConfig {
    #_language: String;
    #_locale: Object;

    constructor() {
        this.#_locale = {};
    }

    async init(pSetup: PaperSetupConfig, languagePackOverride = {}) {
        const { language, paper, customLanguagePackFileTime } = pSetup;
        Logger.debug(`Initializing locale: ${language}, fileTime: ${customLanguagePackFileTime}`);

        this.#_language = language;
        this.#_initLocale();

        LocaleHelper.setStorageName(paper);

        let languagePack = await LocaleHelper.getLanguagePack();
        const lastCustomLanguagePackFileTime = await LocaleHelper.getFiletime();

        if (
            Network.isOnline() &&
            customLanguagePackFileTime > -1 &&
            customLanguagePackFileTime > lastCustomLanguagePackFileTime
        ) {
            try {
                Logger.debug("Downloading new language pack");
                languagePack = await getLanguagePack();
            } catch (e) {
                Logger.warn(e.message);
            }
        } else if (customLanguagePackFileTime === -1) {
            //if no language pack server on the server, reset local settings
            languagePack = {};
        }

        this.#_locale = { ...languagePack, ...languagePackOverride };

        LocaleHelper.setFiletime(customLanguagePackFileTime);
        LocaleHelper.setLanguagePack(this.#_locale);
    }

    #_getLanguage() {
        return this.#_language;
    }

    #_getCurrentLocale() {
        if (this.#_language) {
            if (this.#_language >= 0 && this.#_language < SUPPORTED_LANGUAGES.length) {
                Logger.debug(`Using locale: ${SUPPORTED_LANGUAGES[this.#_language]}`);
                return SUPPORTED_LANGUAGES[this.#_language];
            }
        }
        Logger.debug(`Using default locale: ${SUPPORTED_LANGUAGES[0]}`);
        return SUPPORTED_LANGUAGES[0];
    }

    #_initLocale() {
        i18nInit(this.#_getCurrentLocale());
        DateUtils.initMomentLocale(i18nLocale());
    }

    date(time) {
        return DateUtils.fromNow(time);
    }

    formattedDate(str, format) {
        return DateUtils.fromNow(str, format);
    }

    strings(name, params = {}) {
        let string = this.#_locale[name];
        if (string) {
            string = Object.keys(params).reduce((result, key) => result.replace(`{{${key}}}`, params[key]), string);
        }
        const fallback = i18nTxt(name, params);
        return string || fallback;
    }
}

export default LocaleConfig;
