import React from "react";
import Logger from "../utils/log";
import UserHelper from "../helpers/user-helper";

const SelectedEditionsContext = React.createContext({
    editions: [],
    setEdition: (edition) => {}
});

class UserConfig {
    editions: Array;
    aspectRatio: Number;
    selectedEditions: Array<String>;
    favoriteEdition: String;

    async init(pSetup) {
        Logger.debug("Initializing user settings");
        const { defaultSelectedEditions } = pSetup;
        try {
            const selectedEditions =
                (await this.#_getValue(UserHelper.SP.keys.selected_editions)) || defaultSelectedEditions;
            this.selectedEditions = selectedEditions ? selectedEditions.split(",") : [];
        } catch (e) {
            this.selectedEditions = [];
        }

        try {
            this.editions = JSON.parse(await this.#_getValue(UserHelper.SP.keys.editions));
        } catch (e) {
            Logger.warn("failed to parse editions from action message");
        }

        try {
            this.aspectRatio = parseFloat((await this.#_getValue(UserHelper.SP.keys.aspect_ratio)) || 1);
        } catch (e) {
            Logger.warn("failed to parse aspect ratio");
        }

        this.favoriteEdition = await this.#_getValue(UserHelper.SP.keys.favorite_edition);

        Logger.debug("initialized without errors");
    }

    async #_setValue(key, value) {
        return UserHelper.setItem(UserHelper.SP.name, key, `${value}`);
    }

    async #_getValue(key) {
        let value = this[key];
        if (value === null || value === undefined) {
            Logger.debug(`${key}: ${value}`);
            value = await UserHelper.getItem(UserHelper.SP.name, key);
        }
        return value ?? "";
    }

    addEditionToSelected(editionName: String): Array<String> {
        const index = this.selectedEditions.indexOf(editionName);
        if (index > -1) {
            this.selectedEditions.splice(index, 1);
        } else {
            this.selectedEditions.push(editionName);
        }
        UserHelper.setItem(UserHelper.SP.name, UserHelper.SP.keys.selected_editions, this.selectedEditions.join(","));
        return this.selectedEditions;
    }

    setEditions(editions) {
        if (editions) {
            this.editions = editions;
            this.#_setValue(UserHelper.SP.keys.editions, JSON.stringify(editions));
        }
    }

    setAspectRatio(aspectRatio) {
        this.aspectRatio = aspectRatio;
        this.#_setValue(UserHelper.SP.keys.aspect_ratio, aspectRatio);
    }

    setFavoriteEdition(favoriteEdition) {
        this.favoriteEdition = favoriteEdition;
        this.#_setValue(UserHelper.SP.keys.favorite_edition, favoriteEdition);
    }
}

export { SelectedEditionsContext };
export default UserConfig;
