import LiveConfig from "../data/live-config";
import { TABS } from "../interfaces/navigations";
import Edition from "../models/edition";
import Route from "../models/route";
import RouteAction from "../models/route-action";
import RouteEdition from "../models/route-edition";
import RouteEditionsDate from "../models/route-editions-date";
import RouteTabs from "../models/route-tabs";
import RouteUrl from "../models/route-url";
import Network from "../utils/network";
import { addParamsToURL } from "../utils/strings";
import EditionsHelper from "./editions-helper";

class RoutesHelper {
    /**
     * TODO check also psetup values
     * @param {*} routes
     * @param {*} routeName
     * @returns Route
     */
    static getInitialRoute(routes: Array<Route> = [], routeName: String): Route {
        if (routes.length) {
            for (let i = 0; i < routes.length; i++) {
                if (routes[i] instanceof RouteEdition) {
                    const route: RouteEdition = routes[i];
                    if (route.getEdition().getName() === routeName) {
                        return route;
                    }
                } else if (routes[i] instanceof RouteUrl) {
                    const route: RouteUrl = routes[i];
                    if (route.getRoute() === routeName) {
                        return route;
                    }
                }

                const found = RoutesHelper.getInitialRoute(routes[i].getRoutes(), routeName);
                if (found) {
                    //expand parent of selected route to see selected items layout
                    routes[i].setCollapsed(false);
                    return found;
                }
            }
        }
        return undefined;
    }

    static getUrl(php, customUrl, customUrlTarget) {
        let newUrl = customUrl ? customUrl : "";
        let newTarget = customUrlTarget ? "_blank" : "";
        const params = { ...LiveConfig.urlParams(), pSetup: LiveConfig.getPSetupName(), native: 1 };
        if (php === "ipad_login.php") {
            params.fromOpt = 1;
        }
        if (newUrl) {
            //relative path
            if (newUrl[0] === "/") {
                newUrl = `${LiveConfig.getServer()}${newUrl}`;
            }
            newUrl = Network.addDefaultProtocol(newUrl);
            return { url: newUrl, target: newTarget };
        }
        //custom target disabled by default on HTML5 for puzzles
        const url = addParamsToURL(
            `${LiveConfig.getServer()}${LiveConfig.getPSetup().optionsPageTabPath}${php}`,
            params
        );
        //if the site is HTTPS but the custom link not force to open in a new tab
        if (Network.isHTTPS && url.indexOf("http:") > -1 && !newTarget) {
            newTarget = "_blank";
        }
        return { url, target: newTarget };
    }

    static getMainRoute(editions: Array<Edition>, otherRoutes: Array<Route> = []): RouteTabs {
        const { orderNewsstandTabs } = LiveConfig.getPSetup();

        const route = new RouteTabs(LiveConfig.getLocale().strings("str_newsstand"));
        route.setRoute(TABS.NEWSSTAND);
        const newsstandRoutes = [...RoutesHelper.getEditionRoutes(editions), ...otherRoutes];
        if (orderNewsstandTabs.length > 0) {
            newsstandRoutes.sort((itemA, itemB) => {
                //we support both configuration with route name or display name
                let indexA = orderNewsstandTabs.indexOf(itemA.getRoute());
                if (indexA === -1) {
                    indexA = orderNewsstandTabs.indexOf(itemA.getText());
                }
                let indexB = orderNewsstandTabs.indexOf(itemB.getRoute());
                if (indexB === -1) {
                    indexB = orderNewsstandTabs.indexOf(itemB.getText());
                }
                return indexA - indexB;
            });
        }
        route.setRoutes(newsstandRoutes);

        const { favoriteEdition } = LiveConfig.getSettings();
        if (favoriteEdition) {
            const isFavoriteEditionMain =
                editions.find((edition) => edition.isMain() && edition.getName() === favoriteEdition) !== undefined;
            if (!isFavoriteEditionMain) {
                //if the favorite edition is not a main one we need to search it in other tabs
                newsstandRoutes.forEach((newsstandRoute) => {
                    if (
                        newsstandRoute instanceof RouteEdition &&
                        newsstandRoute.getEdition() &&
                        newsstandRoute.getEdition().getName() === favoriteEdition
                    ) {
                        route.setInitialRouteName(favoriteEdition);
                    }
                });
            }
        }

        return route;
    }

    static getRoutes(tabs: Object, level = 0, parent = ""): Array<Route> {
        const drawerRoutes = [];
        const tabsRoutes = [];
        Object.keys(tabs).forEach((key) => {
            const { action, php, label, customUrl, customUrlTarget, showIfIsLoggedIn, icon, isTabElement } = tabs[key];

            let mRoute;
            if (action) {
                mRoute = new RouteAction();

                mRoute.setAction(action);
            } else if (php || customUrl) {
                mRoute = new RouteUrl();

                const { url, target } = RoutesHelper.getUrl(php, customUrl, customUrlTarget);
                mRoute.setUrl(url);
                mRoute.setTarget(target);
            }

            if (mRoute) {
                mRoute.setRoute(key);
                mRoute.setLabel(label || key);
                mRoute.setIcon(icon);
                mRoute.setLevel(level);
                mRoute.setParent(parent);
                mRoute.setVisibleIfLoggedIn(showIfIsLoggedIn);
                mRoute.setCollapsed(true);

                if (isTabElement) {
                    tabsRoutes.push(mRoute);
                } else {
                    drawerRoutes.push(mRoute);
                }
            }
        });
        return { drawerRoutes, tabsRoutes };
    }

    static makeEditionModel(group: Array<Edition>, name: String) {
        const { favoriteEdition } = LiveConfig.getSettings();

        const mEdition = new Edition();
        mEdition.setGroup(
            group.sort((e1, e2) => {
                if (favoriteEdition && e1.getName() === favoriteEdition) {
                    return -1;
                }
                if (e1.isMain() && e2.isMain()) {
                    return 0;
                }

                return e1.isMain() ? -1 : 1;
            })
        );
        mEdition.setName(name);
        return mEdition;
    }

    static getEditionRoute(config = {}) {
        const { mEdition, routeName, label, fixed = true, index = 0 } = config;
        const mRoute = new RouteEditionsDate();
        mRoute.setRoute(routeName);
        mRoute.setEdition(mEdition);
        mRoute.setLabel(label);
        mRoute.setIssueDayIndex(index);
        mRoute.setFixed(fixed);
        return mRoute;
    }

    static getEditionRoutes(editions: Array<Edition>): Array<RouteEdition> {
        const { newsstandTabs, maxEditionsNotGrouped } = LiveConfig.getPSetup();
        const { favoriteEdition } = LiveConfig.getSettings();

        const routes = [];

        const mainEditions = editions.filter((edition) => edition.isMain());
        const mainEdition =
            (favoriteEdition && editions.find((edition) => edition.getName() === favoriteEdition)) || mainEditions[0];

        newsstandTabs.forEach((tab: String) => {
            if (tab === TABS.LATEST_EDITIONS) {
                routes.push(
                    RoutesHelper.getEditionRoute({
                        mEdition: RoutesHelper.makeEditionModel(editions, "latest_editions_group"),
                        routeName: TABS.LATEST_EDITIONS,
                        label: LiveConfig.getLocale().strings("str_latest")
                    })
                );
            } else if (tab === TABS.CUSTOM_EDITIONS) {
                if (mainEdition) {
                    routes.push(
                        RoutesHelper.getEditionRoute({
                            mEdition: RoutesHelper.makeEditionModel(editions, "custom_editions_group"),
                            routeName: TABS.CUSTOM_EDITIONS,
                            label: LiveConfig.getLocale().strings("str_today"),
                            index: 0,
                            fixed: false
                        })
                    );
                }
            } else if (tab === TABS.PENULTIMATE_EDITIONS) {
                if (mainEdition) {
                    const dayIndex = 1;
                    const dayIssue = mainEdition.getIssues()[dayIndex];
                    //add only if exists something
                    if (dayIssue) {
                        routes.push(
                            RoutesHelper.getEditionRoute({
                                mEdition: RoutesHelper.makeEditionModel(editions, "custom_editions_group"),
                                routeName: TABS.PENULTIMATE_EDITIONS,
                                label: EditionsHelper.getIssueLabel(dayIssue),
                                index: dayIndex
                            })
                        );
                    }
                }
            } else if (tab === TABS.OTHER_EDITIONS) {
                if (maxEditionsNotGrouped > -1 && maxEditionsNotGrouped < editions.length) {
                    const mRoute = new RouteEdition();
                    mRoute.setRoute(TABS.OTHER_EDITIONS);
                    mRoute.setEdition(
                        RoutesHelper.makeEditionModel(
                            editions.filter((item) => !item.isMain()),
                            "max_editions_grouped"
                        )
                    );
                    mRoute.setLabel(LiveConfig.getLocale().strings("grouped_editions"));
                    routes.push(mRoute);
                } else {
                    editions.forEach((edition) => {
                        if (!edition.isMain()) {
                            const mRoute = new RouteEdition();
                            mRoute.setRoute(EditionsHelper.getRouteName(edition.getName()));
                            mRoute.setEdition(edition);
                            mRoute.setLabel(edition.getLabel());
                            routes.push(mRoute);
                        }
                    });
                }
            } else if (tab === TABS.ARCHIVE_EDITIONS) {
                const archiveEditions = [];
                if (mainEdition) {
                    const issues = EditionsHelper.getAllIssuesForMainEditions(editions); //handle multi edition, accept also other main editions
                    for (let i = 0; i < issues.length; i++) {
                        const editionIssues = [];
                        editionIssues.push(
                            issues[i],
                            ...EditionsHelper.getAllOtherIssuesForDate(
                                editions.filter(
                                    (edition) =>
                                        // eslint-disable-next-line implicit-arrow-linebreak
                                        !edition.isMain() &&
                                        !edition.isSecondary() &&
                                        edition.getName() !== mainEdition.getName()
                                ),
                                issues[i].getIssue()
                            )
                        );

                        const edition = new Edition();
                        edition.setIssues(editionIssues);
                        edition.setLabel(issues[i].getDateLabel());
                        edition.setName(issues[i].getEdition());

                        archiveEditions.push(edition);
                    }

                    const mEdition = new Edition();
                    mEdition.setIsMain(true);
                    mEdition.setGroup(archiveEditions);
                    mEdition.setName("archive_editions_group");

                    const archive = new RouteEdition();
                    archive.setEdition(mEdition);
                    archive.setRoute(TABS.ARCHIVE_EDITIONS);
                    archive.setLabel(LiveConfig.getLocale().strings("str_archive_editions"));
                    routes.push(archive);
                }
            } else if (tab === TABS.DOWNLOADED_EDITIONS) {
                const downloadedIssues = EditionsHelper.getDownloadedIssues(editions);
                console.debug(downloadedIssues.length);
                if (downloadedIssues.length) {
                    const mEdition = new Edition();
                    mEdition.setIssues(downloadedIssues);
                    mEdition.setName("downloaded_editions_group");

                    const downloaded = new RouteEdition();
                    downloaded.setEdition(mEdition);
                    downloaded.setRoute(TABS.DOWNLOADED_EDITIONS);
                    downloaded.setLabel(LiveConfig.getLocale().strings("str_downloaded_editions"));
                    routes.push(downloaded);
                }
            }
        });

        return routes;
    }
}

export default RoutesHelper;
