import axios, { Axios } from "axios";
import Issue from "../models/issue";
import Logger from "./log";
import { os } from "./platform";
import { addParamsToURL } from "./strings";

const STUB_TABS = require("../stubs/load-tab-routes.json");

class API {
    static #_server: String;
    static #_entryPoint: String;
    static #_params: Object;
    static #_instance: Object;

    static setServer(value: String) {
        this.#_server = value;
    }

    static getServer(): String {
        return this.#_server;
    }

    static setEntryPoint(value: String) {
        this.#_entryPoint = value;
    }

    static getEntryPoint(): String {
        let server: String = API.getServer();
        if (!server.endsWith("/")) {
            server += "/";
        }

        return `${server}${this.#_entryPoint}`;
    }

    static getPath(): String {
        return this.#_entryPoint;
    }

    static setParams(params: Object = {}) {
        this.#_params = params;
    }

    static getParams(): Object {
        return this.#_params;
    }

    static getDeviceParams(): Object {
        return {
            os: os(),
            isKindle: false
        };
    }

    static init() {
        this.#_instance = axios.create();
    }

    static getInstance(): Axios {
        return this.#_instance;
    }

    static addProtocol(url = "") {
        if (!url.startsWith("http")) {
            return `https://${url}`;
        }
        return url;
    }
}

async function getTabRoutes() {
    return STUB_TABS;
}

function getResUrl(item: Issue, type: String) {
    const siteDomain = item.getSiteDomainCDN() || item.getSiteDomain();
    return addParamsToURL(`${API.addProtocol(siteDomain)}/${API.getPath()}`, {
        ...API.getDeviceParams(),
        ...API.getParams(),
        action: "issueImage",
        type,
        pSetup: item.getPSetup(),
        issue: item.getIssue(),
        crc: item.getFilename(),
        edition: encodeURIComponent(item.getEdition()),
        mtime: item.getLastClosed()
    });
}

function getGraphUrl(item: Issue) {
    return getResUrl(item, "graph");
}

function getTextUrl(item: Issue) {
    return getResUrl(item, "text");
}

function getLowUrl(item: Issue) {
    return getResUrl(item, "bblow");
}

async function get(url, config, defaultValue) {
    try {
        Logger.debug(`GET ${url}`, config);
        const response = await API.getInstance().get(url, config);
        return response.data || defaultValue;
    } catch (e) {
        Logger.warn(`GET ${url} fails with error ${e.message}`);
    }
    return defaultValue;
}

async function getEditions(params = {}) {
    return get(API.addProtocol(API.getEntryPoint()), {
        params: {
            ...API.getDeviceParams(),
            ...API.getParams(),
            action: "editionsList",
            ...params
        }
    });
}

async function getEditionsPics(params = {}) {
    return get(API.addProtocol(API.getEntryPoint()), {
        params: {
            ...API.getDeviceParams(),
            ...API.getParams(),
            action: "editionsPics",
            ...params
        }
    });
}

async function getSpecials(params = {}) {
    return get(API.addProtocol(API.getEntryPoint()), {
        params: {
            ...API.getDeviceParams(),
            ...API.getParams(),
            action: "editionsListMenu",
            ...params
        }
    });
}

async function getPSetup(params = {}) {
    return get(API.addProtocol(API.getEntryPoint()), {
        params: {
            ...API.getDeviceParams(),
            ...API.getParams(),
            action: "message",
            service: "optionspage",
            format: "json",
            ...params
        }
    });
}

async function getLanguagePack(params = {}) {
    return get(API.addProtocol(API.getEntryPoint()), {
        params: {
            ...API.getDeviceParams(),
            ...API.getParams(),
            action: "locale",
            ...params
        }
    });
}

export default API;
export {
    getEditions,
    getEditionsPics,
    getSpecials,
    getTabRoutes,
    getLowUrl,
    getGraphUrl,
    getTextUrl,
    getPSetup,
    getLanguagePack
};
