import { findNodeHandle } from "react-native";
import Logger from "./log";

/**
 * return a promise with timeout, if timeout is called the promise will be rejected
 * @param {*} promiseCall
 * @param {*} promiseTimeout
 * @param  {...any} params
 * @returns
 */
export async function PromiseTimeout(promiseCall: Function, promiseTimeout: Number = 2000, ...params) {
    return new Promise((resolve, reject) => {
        let inTimeout = false;
        setTimeout(() => {
            inTimeout = true;
            reject(new Error("promise timeout"));
        }, promiseTimeout);
        promiseCall(...params)
            .then((data) => {
                if (!inTimeout && typeof resolve === "function") {
                    resolve(data);
                }
            })
            .catch((error) => {
                if (!inTimeout && typeof reject === "function") {
                    reject(error);
                }
            });
    });
}

export default class JSUtils {
    static generateId() {
        // Math.random should be unique because of its seeding algorithm.
        // Convert it to base 36 (numbers + letters), and grab the first 9 characters
        // after the decimal.
        return `${Math.random().toString(36).substr(2, 9)}`;
    }

    static equals(a, b) {
        let resultA;
        let resultB;
        try {
            resultA = JSON.stringify(a);
        } catch (e) {
            //find another way to check the item
            return false;
        }
        try {
            resultB = JSON.stringify(b);
        } catch (e) {
            //find another way to check the item
            return false;
        }
        return resultA === resultB;
    }

    static isValidRect(rect: { start: Number, top: Number, width: Number, height: Number }) {
        return (
            rect &&
            JSUtils.isNumber(rect.start) &&
            JSUtils.isNumber(rect.top) &&
            JSUtils.isNumber(rect.width) &&
            JSUtils.isNumber(rect.height) &&
            rect.width > 0 &&
            rect.height > 0
        );
    }

    static intersectRect(r1, r2) {
        return !(
            r2.start > r1.start + r1.width ||
            r2.start + r2.width < r1.start ||
            r2.top > r1.top + r1.height ||
            r2.top + r2.height < r1.top
        );
    }

    static intersectAreaRect(r1, r2) {
        const intersectionX1 = Math.max(r1.start, r2.start);
        const intersectionX2 = Math.min(r1.start + r1.width, r2.start + r2.width);
        if (intersectionX2 < intersectionX1) {
            return { start: 0, top: 0, width: 0, height: 0 };
        }

        const intersectionY1 = Math.max(r1.top, r2.top);
        const intersectionY2 = Math.min(r1.top + r1.height, r2.top + r2.height);
        if (intersectionY2 < intersectionY1) {
            return { x: 0, y: 0, width: 0, height: 0 };
        }

        return {
            start: intersectionX1,
            top: intersectionY1,
            width: intersectionX2 - intersectionX1,
            height: intersectionY2 - intersectionY1
        };
    }

    static intersectArea(r1, r2) {
        const { width, height } = JSUtils.intersectAreaRect(r1, r2);
        return width * height;
    }

    static rectContainsPoint(r, p) {
        return r.start < p.x && r.start + r.width > p.x && r.top < p.y && r.top + r.height > p.y;
    }

    static inInterval(n, lower, upper) {
        return n >= lower && n <= upper;
    }

    /**
     * check if the first object layer is equal
     * first check the difference from a and b
     * @param {*} a
     * @param {*} b
     */
    static areEqualShallow(a = {}, b = {}, options = {}) {
        const { print = false, skipKeys = [] } = options;
        const aDifferentFromB = Object.keys(a).find((key) => {
            if (typeof a[key] === "function" || typeof a[key] === "object" || skipKeys.includes(key)) {
                return false;
            }
            if (!JSUtils.hasProperty(b, key)) {
                return true;
            }
            if (a[key] !== b[key]) {
                return true;
            }
            return false;
        });
        const bDifferentFromA = Object.keys(b).find((key) => {
            if (typeof b[key] === "function" || typeof b[key] === "object" || skipKeys.includes(key)) {
                return false;
            }
            if (!JSUtils.hasProperty(a, key)) {
                return true;
            }
            return false;
        });
        const areEqual = !aDifferentFromB && !bDifferentFromA;
        if (print && !areEqual) {
            if (aDifferentFromB !== undefined) {
                Logger.debug(`a different from b by ${aDifferentFromB}`);
            }
            if (bDifferentFromA !== undefined) {
                Logger.debug(`b different from a by ${bDifferentFromA}`);
            }
        }
        return areEqual;
    }

    static shouldUpdatePureComponent(currentProps, nextProps, currentState, nextState) {
        let ret = false;
        const prevPropsKeys = Object.keys(currentProps);
        const nextPropsKeys = Object.keys(nextProps);
        if (prevPropsKeys.length !== nextPropsKeys.length) {
            ret = true;
        }
        const prevStateKeys = Object.keys(currentState || {});
        const nextStateKeys = Object.keys(nextState || {});
        if (prevStateKeys.length !== nextStateKeys.length) {
            ret = true;
        }
        // eslint-disable-next-line no-restricted-syntax
        for (const key in currentProps) {
            if (currentProps[key] !== nextProps[key]) {
                ret = true;
            }
        }
        // eslint-disable-next-line no-restricted-syntax
        for (const key in currentState) {
            if (currentState[key] !== (nextState || {})[key]) {
                ret = true;
            }
        }
        return ret;
    }

    static getViewByRef(ref) {
        if (!ref) {
            throw new Error("ref.current is null");
        }
        if (typeof ref === "object" && ref.current) {
            ref = ref.current;
        }
        if (typeof ref !== "number") {
            const node = findNodeHandle(ref);
            if (!node) {
                throw new Error(`findNodeHandle failed to resolve view=${String(ref)}`);
            }
            ref = node;
        }
        return ref;
    }

    static isNull(a) {
        return a === undefined || a === null;
    }

    static isEmpty(a) {
        if (!JSUtils.isNull(a)) {
            if (typeof a === "string") {
                return a.length === 0;
            }
            if (Array.isArray(a)) {
                return a.length === 0;
            }
            if (typeof a === "object") {
                return Object.keys(a).length === 0;
            }
        }
        return true;
    }

    static hasProperty(obj = {}, property = "") {
        return Object.prototype.hasOwnProperty.call(obj, property);
    }

    static isNumber(value) {
        try {
            return !(typeof value !== "number" || Number.isNaN(value));
        } catch (e) {
            return false;
        }
    }

    static max(...args) {
        return Math.max(...args.filter((arg) => JSUtils.isNumber(arg)));
    }

    static min(...args) {
        return Math.min(...args.filter((arg) => JSUtils.isNumber(arg)));
    }

    static iterator(iterator, callback) {
        if (iterator && typeof iterator.next === "function") {
            let iterate;
            let index = 0;
            do {
                iterate = iterator.next();
                if (typeof callback === "function" && iterate.value !== undefined) {
                    const [key, value] = iterate.value;
                    callback({ key, value }, index);
                }
                index += 1;
            } while (!iterate.done);
        }
    }
}
